#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s):
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: biomodels.py 141 2013-02-06 10:22:25Z cokelaer $

"""This module provides a class :class:`ChEMBLdb` 

.. topic:: What is ChEMBLdb

    :URL:  https://www.ebi.ac.uk/chembldb/index.php/
    :REST: https://www.ebi.ac.uk/chembldb/index.php/ws
    :FAQS: https://www.ebi.ac.uk/chembldb/index.php/faq


    .. highlights::

        "Using the ChEMBLdb web service API users can retrieve data from the ChEMBL
        database in a programmatic fashion. The following list defines the currently
        supported functionality and defines the expected inputs and outputs of each
        method."

        -- From ChEMBLdb web page Dec 2012




"""
import urllib2, urllib, json, re, os
from bioservices.services import *
import webbrowser


class ChEMBLdb(RESTService):
    """Interface to `ChEMBLdb <http://www.ebi.ac.uk/chembldb/index.php>`_ 


    Here is a quick example to retrieve a target given its ChEMBL Id
    .. doctest::

        >>> from bioservices import ChEMBLdb
        >>> s = ChEMBLdb(verbose=False)
        >>> print(s._target_chemblId_example)
        'CHEMBL2477'
        >>> resjson = s.get_target_by_chemblId(s._target_chemblId_example+".json")

    """
    _url = "http://www.ebi.ac.uk/chemblws/"
    _chemblId_example = "CHEMBL1"
    _inChiKey_example = "QFFGVLORLPOAEC-SNVBAGLBSA-N"
    _smiles_example = "COc1ccc2[C@@H]3[C@H](COc2c1)C(C)(C)OC4=C3C(=O)C(=O)C5=C4OC(C)(C)[C@@H]6COc7cc(OC)ccc7[C@H]56"
    _smiles_similar_example = _smiles_example + "/90"
    _image_chemblId_example = "CHEMBL192"
    _image_dimension_example = 200
    _bioactivities_example = "CHEMBL2"
    _target_chemblId_example = "CHEMBL2477"
    _target_uniprotId_example = "Q13936"
    _target_bioactivities_example = "CHEMBL240"
    _assay_example = "CHEMBL1217643"
    _inspect_example = ('CHEMBL1','compound')

    def __init__(self, verbose=True):
        super(ChEMBLdb, self).__init__(url=ChEMBLdb._url, 
            name="ChEMBLdb", verbose=verbose)
        self._default_extension = "json"

    # just a get/set to the default extension
    def _set_default_ext(self, ext):
        self.checkParam(ext, ["json","xml"])
        self._default_extension = ext
    def _get_default_ext(self):
        return self._default_extension
    default_extension = property(_get_default_ext, _set_default_ext, 
        doc="set extension of the requests (default is json). Can be 'json' or 'xml'")

    # ChEMBL specifies those errors, so let us try to catch thmem. May not be
    # needed. Could to everything within services module.
    def _request(self, query, format="xml", baseUrl=True):
        """calls RESTService.request and check for specific Chembl errors.

        If error are 400, 404 or 500 then a specific Chembl error message is
        returned.
        """
        try:
            res = super(ChEMBLdb, self).request(query, format, baseUrl)
            return res
        except urllib2.HTTPError,e:
            if e.code == 400:
                self.logging.error(""" Bad request. The parameters passed to the
API endpoint were deemed invalid. This response will be returned for invalid
ChEMBLId's i.e. CHEMBLX1, invalid UniProt accessions, invalid SMILES strings
e.t.c. """)
            elif e.code == 404:
                self.logging.error(""" Not found. The resource corresponding to
the supplied parameters does not exist. This response will be returned for
requests for non-existent ChEMBL compound, target, and assay resources. """)
            elif e.code == 500:
                self.logging.error(""" Service unavailable. An internal problem
prevented us from fulfilling your request. """)
            else:
                pass
            raise BioServicesError("Unknown error caught (not 400,404, 500)")
        except Exception,e:
            raise(e)


    def api_status(self):
        """ Check API status

        :return: Response is the string 'UP' if services are running
        """
        res = self.request("status/", format="txt")
        return res

    # a complicated decorator that simplifies the code significantly
    # can accept parameters
    # can be used by methods
    def __process(*args_deco, **kwds_deco):
        service = args_deco[0]
        if len(args_deco)>1:
            service_extra = args_deco[1]
        else:
            service_extra = None

        def decorator(func):
            def newf(self, query):
                """
                There are two cases: query is a string or a list/tuple of strings.
                In the first case, query is something close to url query. In the second case,
                is a family of those ones.
                """
                def infunc(query):
                    if query.endswith(".json"):
                        output = "json"
                    elif query.endswith(".xml"):
                        output = "xml"
                    elif len(query.split("."))>=2:
                        raise BioServicesError("""The query must be a valid identifier with .json or .xml extensions or no extension at all (default XML)""")
                    else: # query has no extension, so we need to figure out the default
                        output = self.default_extension
                        query += "." +  self.default_extension
                    if service_extra:
                        # used for example by get_compounds_activities
                        url = self.url + "/" + service + "/" + query.split('.')[0]
                        url += "/" + service_extra
                        if output == "json":
                            url += ".json"
                    else:
                        url = self.url + "/" + service + "/" + query
                    self.logging.info(url)
                    # the request itself
                    res = self._request(url, output)

                    # converion required if json structure.
                    if output == "json":
                        res = json.loads(res)
                    return res
                if isinstance(query,str):
                    res = infunc(query)
                elif isinstance(query,list) or isinstance(query,tuple):
                    res = tuple(map(infunc, query))
                else:
                    raise TypeError("query must be a string, list or tuple!!")
                return res
                #return func(self, *args, **kwds)
            newf.__name__ = func.__name__
            newf.__doc__ = func.__doc__
            return newf
        return decorator


    @__process("compounds")
    def get_compounds_by_chemblId(self, query):
        """Get compound by ChEMBLId

        :param query: a valid compound ChEMBLId or a list of those ones.
            A ".json" or ".xml" extension can be added to bypass default
            :attr:`default_extension`.
        :return: Compound Record in XML or dictionary (if json requested). If
            query os a list/tuple, a  tuple of compound records is returned.

        If json format is requested, a dictionary is returned. The dictionary
        has a unique key 'compound'. The value of that key is another dictionary
        keyed by:

         * acdLogd
         * acdLogp
         * alogp
         * chemblId
         * knownDrug
         * medChemFriendly
         * molecularFormula
         * molecularWeight
         * numRo5Violations
         * passesRuleOfThree
         * rotatableBonds
         * smiles
         * stdInChiKey

        :: 

            >>> from bioservices import *   
            >>> s = ChEMBLdb(verbose=False)
            >>> print (s._chemblId_example)
            >>> resxml = s.get_compounds_by_chemblId(s._chemblId_example)
            >>> resjson = s.get_compounds_by_chemblId(s._chemblId_example)
        """
        # the decorator is taking care of the checking and processing
        pass
 

    @__process("compounds/stdinchikey")
    def get_individual_compounds_by_inChiKey(self, query):
        """Get individual compound by standard InChi Key

        :param str query: a valid InChi key or a list of those ones.
            A ".json" or ".xml" extension can be added to bypass default
            :attr:`default_extension`.
        :return: Compound Record in XML or dictionary (if json requested). If
            query os a list/tuple, a  tuple of compound records is returned.

        In addition to the keys returned in :meth:`get_compounds_by_chemblId`,
        the following keys are returned:

         * acdAcidicPka
         * acdBasicPka
         * preferredCompoundName
         * species
         * synonyms

        .. doctest::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._inChiKey_example)
            QFFGVLORLPOAEC-SNVBAGLBSA-N
            >>> resxml = s.get_individual_compounds_by_inChiKey(s._inChiKey_example + ".xml")
            >>> resjson = s.get_individual_compounds_by_inChiKey(s._inChiKey_example + ".json")
        """
        # the decorator is taking care of the checking and processing
        pass

    @__process("compounds/smiles")
    def get_compounds_by_SMILES(self, query):
        """Get list of compounds by Canonical SMILES

        :param str query: a valid compound ChEMBLId or a list of those ones.
            A ".json" or ".xml" extension can be added to bypass default
            :attr:`default_extension`.
        :return: Compound Record in XML or dictionary (if json requested). If
            query os a list/tuple, a  tuple of compound records is returned.

        If json format is requested, a dictionary is returned. The dictionary
        has a unique key 'compounds'. The value of that key is a list of compound
        records. For each compound record dictionary see :meth:`get_compounds_by_chemblId`.

        ::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._smiles_example)
            >>> resxml = s.get_compounds_by_SMILES(s._smiles_example + ".xml")
            >>> resjson = s.get_compounds_by_SMILES(s._smiles_example + ".json")
        """
        pass


    @__process("compounds/substructure")
    def get_compounds_containing_SMILES(self, query):
        """Get list of compounds containing the substructure represented
        by the given Canonical SMILES

        :param str query: a valid SMILES string or a list of those ones.
            A ".json" or ".xml" extension can be added to bypass default
            :attr:`default_extension`.
        :return: see :meth:`get_compounds_by_SMILES`


        ::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._smiles_example)
            >>> resxml = s.get_compounds_containing_SMILES(s._smiles_example + ".xml")
            >>> resjson = s.get_compounds_containing_SMILES(s._smiles_example + ".json")
        """
        pass


    @__process("compounds/similarity")
    def get_compounds_similar_to_SMILES(self, query):
        """Get list of compounds similar to the one represented by the given Canonical SMILES.

        The similarity is at a cutoff percentage score (minimum value=70%, maximum value=100%).

        :param str query: a valid SMILES string or a list of those ones.
            A ".json" or ".xml" extension can be added to bypass default
            :attr:`default_extension`.
            The SMILE string must be followed by a slash character and the
            expected similarity (e.g., "/90").
        :return: Compound records. See :meth:`get_compounds_by_chemblId`

        In addition to the keys returned in :meth:`get_compounds_by_chemblId`,
        the following keys are returned:

         * acdAcidicPka
         * acdBasicPka
         * similarity
         * species

        ::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._smiles_similar_example)
            >>> resjson = s.get_compounds_similar_to_SMILES(s._smiles_example + "/100")
            >>> resjson = s.get_compounds_similar_to_SMILES(s._smiles_similar_example)
        """
        pass


    def get_image_of_compounds_by_chemblId(self, query, dimensions=None,
            file_out=True, view=True):
        """Get the image of a given compound with png fromat.

        :param str query: a valid compound ChEMBLId or a list/tuple of valid compound ChEMBLIds.
        :param int dimensions: optional argument. An integer z such that :math:`1 \leq z \leq 500`
            giving the dimensions of the image.
            If query is a list and dimension is None or an integer, a list of the same size than query
            is created containing the same initial value into all positions.
            If both query and diemnsion are lists, the ith dimension is related to the image of the ith query.
        :param file_out: Can be True | False | a string | list of strings. If
            True, the images are automatically saved to home current work directory with the
            name of compound.
            If False, the images are not saved.
            If string and query is also a string, the image is saved to the path contained into the string.
            If string and query is a list, a list of paths is created using file_out as a seed.
            If list with different length of query, is completed with False's to be the same length
            If both file_out and query are lists with the same length, the image corresponding to ith query
            is saved using ith path into fiel_out.
        :param bool view: show the image. If True the images are opened.
        :return: the path (list of paths) used to save the figure (figures) (different from Chembl API)


        ::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print (s._image_chemblId_example, s._image_dimension_example)
            >>> s.get_image_of_compounds_by_chemblId(s._image_chemblId_example)
            >>> s.get_image_of_compounds_by_chemblId(s._image_chemblId_example, s._image_dimension_example)
        """        
        def check_args(master_arg, slave_arg, master_name, slave_name):
            """
            It checks if the slave_arg is agree with master_arg
            """
            if slave_name == 'file_out':
                if isinstance(master_arg,str):
                    if not isinstance(slave_arg,str) and not isinstance(slave_arg,bool):
                        raise TypeError("%s type must be a str or bool type."%slave_name)
                elif isinstance(master_arg,list) or isinstance(master_arg, tuple):
                    if not all([isinstance(v,str) for v in master_arg]):
                        raise TypeError("All %s types must be str type."%master_name)
                    Lm = len(master_arg)
                    if isinstance(slave_arg,bool):
                        slave_arg = [slave_arg]*len(Lm)
                    elif isinstance(slave_arg,str):
                        slave_arg = [slave_arg + '-%d'%i for i in xrange(Lm)]
                    elif isinstance(slave_arg,list) or isinstance(slave_arg,tuple):
                        if len(slave_arg) > len(master_arg):
                            slave_arg = slave_arg[:Lm]
                        if not all([any([isinstance(v,t) for t in [str, bool]]) for v in slave_arg]):
                            raise TypeError("All %s types must be str or bool type."%slave_name)
                        Ls = len(slave_arg)
                        slave_arg = list(slave_arg) + [False for i in xrange(Lm-Ls)]
                    else:
                        raise TypeError("%s type must be a str, bool, list or tuple type"%slave_name)
                else:
                    raise TypeError("%s type must be a str, list or tuple type"%master_name)
                return slave_arg
            elif slave_name == 'dimensions':
                if isinstance(master_arg,str):
                    if not type(slave_arg) == int and not isinstance(slave_arg, type(None)):
                        raise TypeError("%s type must be a int or NoneType type."%slave_name)
                elif isinstance(master_arg,list) or isinstance(master_arg, tuple):
                    if not all([isinstance(v,str) for v in master_arg]):
                        raise TypeError("All %s types must be str type."%master_name)
                    Lm = len(master_arg)
                    if isinstance(slave_arg,type(None)):
                        slave_arg = [slave_arg]*Lm
                    elif type(slave_arg) == int:
                        slave_arg = [slave_arg]*Lm
                    elif isinstance(slave_arg,list) or isinstance(slave_arg,tuple):
                        if len(slave_arg) > len(master_arg):
                            slave_arg = slave_arg[:Lm]
                        if not all([any([type(v) == t for t in [int, type(None)]]) for v in slave_arg]):
                            raise TypeError("All %s types must be int or NoneType type."%slave_name)
                        Ls = len(slave_arg)
                        slave_arg = list(slave_arg) + [None]*(Lm-Ls)
                    else:
                        raise TypeError("%s type must be a str, bool, list or tuple type"%slave_name)
                else:
                    raise TypeError("%s type must be a str, list or tuple type"%master_name)
                return slave_arg

        def __f_save(target_data,file_out):
            FILE = open(file_out,'w')
            FILE.write(target_data)
            FILE.close()
            print "saved to %s"%file_out
        file_out = check_args(query,file_out,'query','file_out')
        dimensions = check_args(query,dimensions,'query','dimensions')
        if isinstance(query, str):
            url='http://www.ebi.ac.uk/chemblws/compounds/%s/image' % query
            if dimensions is not None:
                url += '?dimensions=%s'%dimensions
            target_data = urllib2.urlopen(url).read()
            print url
            if file_out is True:
                file_out = os.getcwd()
                file_out += '/%s.png' % query
            if not file_out is False:
                __f_save(target_data,file_out)
            if view:
                webbrowser.open(file_out)
            fout = file_out
        elif isinstance(query, tuple) or isinstance(query,list):
            L = len(query)
            if not file_out is False:
                fout = []
            for i in xrange(L):
                item = query[i]
                fo = file_out[i]
                dim = dimensions[i]
                url= self.url + '/compounds/%s/image' % item
                if dim is not None:
                    url += '?dimensions=%s'%dim
                target_data = urllib2.urlopen(url).read()
                if fo is False:
                    fo = os.getcwd()
                    fo += '/%s.png'%item
                if not fo is False:
                    fout.append(fo)
                    __f_save(target_data,fo)
                if view:
                    webbrowser.open(fo)                
        return fout


    @__process("compounds", "bioactivities")
    def get_compounds_activities(self, query):
        """Get individual compound bioactivities

        :param str query: Compound ChEMBLId with optional extension. This is different from
            the API!! API is compounds/%s/bioactivities.json here we need to provide json if
            we want json output. So query must be CHEMBL2.json (or use
            default_extension
        :return: Bioactivity records in XML or dictionary (if json requested)

        If json format is requested, a dictionary is returned. The dictionary
        has a unique key 'bioactivities'. The value of that key is a lsit of bioactivity
        records. Each bioactivity record is a dictionary keyed by:

         * activity_comment
         * assay_chemblid
         * assay_description
         * assay_type
         * bioactivity_type
         * ingredient_cmpd_chemblid
         * name_in_reference
         * operator
         * organism
         * parent_cmpd_chemblid
         * reference
         * target_chemblid
         * target_confidence
         * target_name
         * units
         * value

        ::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._bioactivities_example)
            >>> resxmls = s.get_compounds_activities(s._bioactivities_example + '.xml')
            >>> resjson = s.get_compounds_activities(s._bioactivities_example)
        """
        pass


    @__process("targets")
    def get_target_by_chemblId(self, query):
        """Get target by ChEMBLId

        :param str query: a target ChEMBLId. A ".json" or ".xml" extension
            can be added to bypass default :attr:`default_extension`
        :param str query: target ChEMBLId
        :return: Target Record in XML or dictionary (if json requested)

        If json format is requested, a dictionary is returned. The dictionary
        has a unique key 'target'. The value of that key is another dictionary
        keyed by:

         * target
         * description
         * geneNames
         * organism
         * preferredName
         * proteinAccession
         * synonyms
         * targetType

        ::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._target_chemblId_example)
            >>> resxml = s.get_target_by_chemblId(s._target_chemblId_example + '.xml')
            >>> resjson = s.get_target_by_chemblId(s._target_chemblId_example)
        """
        pass


    @__process("targets/uniprot")
    def get_target_by_uniprotId(self, query):
        """Get individual target by UniProt Accession Id

        :param str query: a valid uniprot accession Id. A ".json" or ".xml" extension
            can be added to bypass default :attr:`default_extension`
        :return: see :meth:get_target_by_chemblId

        ::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._target_chemblId_example)
            >>> resxmls = s.get_target_by_uniprotId(s._target_uniprotId_example + '.xml')
            >>> resjson = s.get_target_by_uniprotId(s._target_uniprotId_example)
        """
        pass


    @__process("targets/refseq")
    def get_target_by_refSeqId(self, query):
        """Get individual target by RefSeq Accession Id

        .. warning:: not yet working. Example on the Chembl website is not working.

        :param str query: a valid RefSea accession Id
        :return: Target Record


        .. existing one: NP_015325
        """
        pass

    @__process("targets","bioactivities")
    def get_target_bioactivities(self, query):
        """Get individual target bioactivities

        :param str query: a valid target ChEMBLId. A ".json" or ".xml" extension
            can be added to bypass default :attr:`default_extension`
        :return: see :meth:`get_compounds_activities`

        ::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._target_bioactivities_example)
            >>> resxml = s.get_target_bioactivities(s._target_bioactivities_example + '.xml')
            >>> resjson = s.get_target_bioactivities(s._target_bioactivities_example)
        """
        pass


    def get_all_targets(self):
        """Get all targets in a dictionary

        :return: Target Record in a dictionary (default is json)
          

        The attribute :attr:`default_extension` is set to json so that
        the returned objetc is a json format. You may set the attribute to
        another format such as "XML"
        
        The returned dictionary has a unique key called **targets**.
        The value of that key is a list of dictionaries with the following keys:


          * chemblId
          * description
          * geneNames
          * organism
          * preferredName
          * proteinAccession
          * synonyms
          * targetType

        ::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> res = s.get_all_targets()
        """
        if self.default_extension == "json":
            res = self.request(self.url + "/targets." + self.default_extension, format="json")
            res = json.loads(res)
        else:
            res = self.request(self.url + "/targets." + self.default_extension, format="json")
        return res


    @__process("assays")
    def get_assay_by_chemblId(self, query):
        """Get assay by ChEMBLId

        :param str query: a valid assay ChEMBLId. A ".json" or ".xml" extension
            can be added to bypass default :attr:`default_extension`
        :return: Assay record in XML or dictionary (if json requested)

        If json format is requested, a dictionary is returned. The dictionary
        has a unique key 'assay'. The value of that key is another dictionary
        keyed by:

         * assayDescription
         * assayOrganism
         * assayStrain
         * assayType
         * chemblId
         * journal
         * numBioactivities

        .. doctest::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._assay_example)
            CHEMBL1217643
            >>> resxml = s.get_assay_by_chemblId(s._assay_example + '.xml')
            >>> resjson = s.get_assay_by_chemblId(s._assay_example)

        """
        pass


    @__process("assays", "bioactivities")
    def get_assay_bioactivities(self, query):
        """Get individual assay bioactivities

        :param str query: a valid assay ChEMBLId. A ".json" or ".xml" extension
            can be added to bypass default :attr:`default_extension`
        :return: Bioactivity records for a given assay in XML or dictionary
            (if json requested). See :meth:`get_compounds_activities`

        ::

            >>> from bioservices import *
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._assay_example)
            >>> resxml = s.get_assay_bioactivities(s._assay_example + '.xml')
            >>> resjson = s.get_assay_bioactivities(s._assay_example)
        """
        pass


    def inspect(self, query, item_type):
        """Open the URL of a query in a browser.

        :param str query: a valid ChEMBLId of a compound, target or assay.
        :param str item_type: a valid type. Might be compound, target or assay

        ::

            >>> from bioservices import *   
            >>> s = ChEMBLdb(verbose=False)
            >>> print(s._assay_example)
            >>> s.inspect(*s._inspect_example)
            >>> print(s._assay_example)
            >>> s.inspect(s._assay_example,'assay')

        """
        url = "https://www.ebi.ac.uk/chembldb/%s/inspect/%s"%(item_type, query)
        webbrowser.open(url)



# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2012 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://www.ebi.ac.uk/~cokelaer/bioservices
#
##############################################################################
"""Interface to the MIRIAM Web Service.

.. topic:: What is Miriam

    :URL: http://www.ebi.ac.uk/miriam/main/
    :WSDL: http://www.ebi.ac.uk/miriamws/main/MiriamWebServices?wsdl

    .. highlights::

        The MIRIAM Registry provides a set of online services for the generation of unique and perennial identifiers, in the form of URIs. It provides the core data which is used by the Identifiers.org resolver.
        The core of the Registry is a catalogue of data collections (corresponding to controlled vocabularies or databases), their URIs and the corresponding physical URLs or resources. Access to this data is made available via exports (XML) and Web Services (SOAP). 

        -- From MIRIAM Web Site, Feb 2013

:Terminology:

* URI: Uniform Resource Identifiers
* URL: Uniform Resource Locators

.. testsetup:: miriam

    from bioservices import *
    m = Miriam()


"""
from bioservices import WSDLService
import webbrowser
import copy

__all__ = ["Miriam"]


class Miriam(WSDLService):
    """Interface to the `MIRIAM <http://www.ebi.ac.uk/miriam/main/>`_ service

    ::

        >>> from bioservices import Miriam
        >>> m = Miriam()
        >>> m.getMiriamURI("http://www.ebi.ac.uk/chebi/#CHEBI:17891")
        'urn:miriam:chebi:CHEBI%3A17891'


    """
    def __init__(self, verbose=True):
        """.. rubric:: Constructor

        :param bool verbose:
        :param bool debug:
        :param str url: redefine the wsdl URL 

        """
        url = "http://www.ebi.ac.uk/miriamws/main/MiriamWebServices?wsdl"
        super(Miriam, self).__init__(name="Miriam", url=url, verbose=verbose)

    def _boolean_convertor(self, boolean):
        if boolean == 'true':
            return True
        elif boolean == 'false':
            return False
        else:
            raise ValueError("boolean can be true or false only (%s)" % boolean)

    def checkRegExp(self, identifier, datatype):
        """Checks if the identifier given follows the regular expression of the data collection

        :param str identifier: internal identifier used by the data collection
        :param str datatype: name, synonym or MIRIAM URI of a data collection
        :return: True if the identifier follows the regular expression, False otherwise

        ::

            >>> m.checkRegExp("P62158", "uniprot")
            True
            >>> m.checkRegExp("!P62158", "uniprot")
            False

        .. warning:: there is no sanity check on the datatype. Default output is
            True. So if you inverse the parameters, you may get True even tough it does not
            make sense. This is a feature of the Web Service.
        """
        res = self.serv.checkRegExp(identifier, datatype)
        res = self._boolean_convertor(res)
        return res

    def convertURL(self, url):
        """Converts an Identifiers.org URL into its equivalent MIRIAM URN.

        This performs a check of the identifier based on the recorded regular expression.
        :param str identifier: an Identifiers.org URL
        :return: the MIRIAM URN corresponding to the provided Identifiers.org URL or 'null' if the provided URL is invalid 

        ::

            >>> m.convertURL("http://identifiers.org/ec-code/1.1.1.1")
            'urn:miriam:ec-code:1.1.1.1'

        """
        res = self.serv.convertURL(url)
        return res

    def convertURLs(self, urls):
        """Converts a list of Identifiers.org URLs into their equivalent MIRIAM URNs.

        This performs a check of the identifier based on the recorded regular expression. 

        :param str identifier: a list of Identifiers.org URL
        :return: the MIRIAM URN corresponding to the provided Identifiers.org URL or 'null' if the provided URL is invalid 

        ::

            >>> m.convertURLs(['http://identifiers.org/pubmed/16333295', 'http://identifiers.org/ec-code/1.1.1.1"])


        """
        res = self.serv.convertURLs(urls)
        return res

    def convertURN(self, urn):
        """Converts a MIRIAM URI into its equivalent Identifiers.org URL.

        This takes care of any necessary conversion, for example in the case 
        the URI provided is obsolete.

        :param str urn: a MIRIAM URLs
        :return: the Identifiers.org URL corresponding to the provided MIRIAM 

        URI or None if the provided URI does not exist 

        ::
    
            >>> m.serv.convertURN('urn:miriam:ec-code:1.1.1.1')
            'http://identifiers.org/ec-code/1.1.1.1'

        """
        res = self.serv.convertURN(urn)
        return res

    def convertURNs(self, urns):
        """Converts a list of MIRIAM URIs into their equivalent Identifiers.org URLs.

        This takes care of any necessary conversion, for example in the case a 
        URI provided is obsolete. If the size of the list of URIs exceeds 200, 
        None is returned.

        :param list urns: list of MIRIAM URIs
        :return: a list of Identifiers.org URLs corresponding to the provided URIs 

        ::

            >>> m.serv.convertURN(['urn:miriam:ec-code:1.1.1.1'])
            ['http://identifiers.org/ec-code/1.1.1.1']

        """
        res = self.serv.convertURNs(urns)
        return res

    def getDataResources(self, nickname):
        """Retrieves all the physical locations (URLs) of the services providing the data collection (web page).

        :param str nickname: name (can be a synonym) or URL or URN of a data collection name (or synonym) or URI (URL or URN)

        :return: array of strings containing all the address of the main page of the resources of the data collection

        ::

            >>> m.getDataResources("uniprot")
            ['http://www.ebi.uniprot.org/', 'http://www.pir.uniprot.org/', 'http://www.expasy.uniprot.org/', 'http://www.uniprot.org/', 'http://purl.uniprot.org/', 'http://www.ncbi.nlm.nih.gov/protein/']
 
        """
        res = self.serv.getDataResources(nickname)
        return res


    def getDataTypeDef(self, nickname):
        """Retrieves the definition of a data collection.

        :param str nickname: name or URI (URN or URL) of a data collection
        :return: definition of the data collection

        ::

            >>> m.getDataTypeDef("uniprot")
            'The UniProt Knowledgebase (UniProtKB) is a comprehensive resource for protein sequence and functional information with extensive cross-references to more than 120 external databases. Besides amino acid sequence and a description, it also provides taxonomic data and citation information.'

        """
        res = self.serv.getDataTypeDef(nickname)
        return res

    def getDataTypePattern(self, nickname):
        """Retrieves the pattern (regular expression) used by the identifiers within a data collection.

        :param str nickname: data collection name (or synonym) or URI (URL or URN)
        :return:  pattern of the data collection 

        ::

            >>> m.getDataTypePattern("uniprot")
            '^([A-N,R-Z][0-9][A-Z][A-Z, 0-9][A-Z, 0-9][0-9])|([O,P,Q][0-9][A-Z, 0-9][A-Z, 0-9][A-Z, 0-9][0-9])(\\.\\d+)?$'
        """
        res = self.serv.getDataTypePattern(nickname)
        return res

    def getDataTypeSynonyms(self, name):
        """Retrieves all the synonym names of a data collection (this list includes the original name).

        :param str name: name or synonym of a data collection
        :return: all the synonym names of the data collection (list of strings)

        ::

            >>> m.getDataTypeSynonyms("uniprot")
            ['UniProt Knowledgebase', 'UniProtKB', 'UniProt']


        """
        res = self.serv.getDataTypeSynonyms(name)
        return res

    def getDataTypeURI(self, name):
        """Retrieves the unique (official) URI of a data collection (example: "urn:miriam:uniprot").

        :param str name: name or synonym of a data collection (examples: "UniProt")
        :return: unique URI of the data collection 

        ::

            >>> m.serv.getDataTypeURI("uniprot")
            'urn:miriam:uniprot'

        """
        res = self.serv.getDataTypeURI(name)
        return res

    def getDataTypeURIs(self, name):
        """Retrieves all the URIs of a data collection, including all the deprecated ones (examples: "urn:miriam:uniprot", "http://www.uniprot.org/", "urn:lsid:uniprot.org:uniprot", ...).

        :param str name: name (or synonym) of the data collection (examples: "ChEBI", "UniProt")
        :return:  all the URIs of a data collection (including the deprecated ones)

        ::

            >>> m.getDataTypeURIs("uniprot")
            ['urn:miriam:uniprot', 'urn:lsid:uniprot.org:uniprot', 'urn:lsid:uniprot.org', 'http://www.uniprot.org/']

        """
        res = self.serv.getDataTypeURIs(name)
        return res

    def getDataTypesId(self):
        """Retrieves the internal identifier (stable and perennial) of all the data collections (for example: "MIR:00000005").


        :return: list of the identifier of all the data collections 

        ::

            >>> m.getDataTypesId()

        """
        res = self.serv.getDataTypesId()
        return res

    def getDataTypesName(self):
        """Retrieves the list of names of all the data collections available.

        :return: list of the name of all the data collections

        ::

            >>> m = self.getDataTypesName()

        """
        res = self.serv.getDataTypesName()
        return res

    def getJavaLibraryVersion(self):
        res = self.serv.getJavaLibraryVersion()
        return res

    def getLocation(self, uri, resource):
        """Retrieves the physical location (URL) of a web page providing knowledge about a specific entity, using a specific resource.

        :param str uri: MIRIAM URI of an entity (example: 'urn:miriam:obo.go:GO%3A0045202')
        :param sr resource: internal identifier of a resource (example: 'MIR:00100005')
        :return: physical location of a web page providing knowledge about the given entity, using a specific resource

        ::

            >>> m.getLocation("UniProt", "MIR:00100005")
            ['http://www.uniprot.org/uniprot/P62158', 'http://purl.uniprot.org/uniprot/P62158', 'http://www.ncbi.nlm.nih.gov/protein/P62158']

        .. warning:: getLocation needs a proper example
        """
        res = self.serv.getLocation(uri, resource)
        return res

    def getLocations(self, nickname, Id=None):
        """Retrieves the (non obsolete) physical locationS (URLs) of web pageS providing knowledge about an entity.


        :param str nickname: name (can be a synonym) or URI of a data collection (examples: "Gene Ontology", "UniProt"). If Id is None, nickname parameter is a MIRIAM URI of an entity (example: 'urn:miriam:obo.go:GO%3A0045202')
        :param str id: identifier of an entity within the given data collection (examples: "GO:0045202", "P62158"). 
        :return:  physical locationS of web pageS providing knowledge about the given entity. If the URI is not recognised or the data collection does not exist, an empty array is returned. If the identifier is invalid for the data collection, None is returned. All special characters in the data entry part of the URLs are properly encoded.

        ::

            >>> m.getLocations("UniProt","P62158")
            >>> m.getLocations("urn:miriam:obo.go:GO%3A0045202")

        """
        if Id:
            res = self.serv.getLocations(nickname, Id)
        else:
            res = self.serv.getLocations(nickname)
        return res

    def getLocationsWithToken(self, nickname, token):
        """Retrieves the list of (non obsolete) generic physical locations (URLs) of web pageS providing the dataset of a given data collection.

        :param str nickname: name (can be a synonym) or URI of a data collection (examples: "Gene Ontology", "UniProt", "urn:miriam:biomodels.db")
        :param str token: placeholder which will be put in the URLs at the location of the data entry identifier (default: $id)
        :return: list of (non obsolete) generic physical locations (URLs) of web pageS providing the dataset of a given data collection

        ::

            >>> m.getLocationsWithToken("uniprot", "P43403")

        """
        res = self.serv.getLocationsWithToken(nickname, token)
        return res

    def getMiriamURI(self, name):
        """Transforms a MIRIAM URI into its official equivalent (to transform obsolete URIs into current valid ones).


        :param str uri: deprecated URI (URN or URL), example: "http://www.ebi.ac.uk/chebi/#CHEBI:17891"
        :return: the official URI corresponding to the deprecated one (for example: "urn:miriam:obo.chebi:CHEBI%3A17891") or 'null' if the URN does not exist 

        ::

            >>> m.getMiriamURI("http://www.ebi.ac.uk/chebi/#CHEBI:17891")
            'urn:miriam:chebi:CHEBI%3A17891'

        """
        res = self.serv.getMiriamURI(name)
        return res

    def getName(self, uri):
        """Retrieves the common name of a data collection

        :param str uri: URI (URL or URN) of a data collection
        :return:  the common name of the data collection

        ::

            >>> m.getName('urn:miriam:uniprot')
            'UniProt Knowledgebase'
 
        """
        res = self.serv.getName(uri)
        return res


    def getNames(self, uri):
        """Retrieves all the names (with synonyms) of a data collection.

        :param str uri:  	URI (URL or URN) of a data collection
        :return: the common name of the data collection and all the synonyms
    
        ::

            >>> m.serv.getNames('urn:miriam:uniprot')
            ['UniProt Knowledgebase', 'UniProtKB', 'UniProt']

        """
        return self.serv.getNames(uri)
    
    def getOfficialDataTypeURI(self, nickname):
        """Retrieves the official URI (it will always be a URN) of a data collection.

        :param str nickname: name (can be a synonym) or MIRIAM URI (even deprecated one) of a data collection (for example: "ChEBI", "http://www.ebi.ac.uk/chebi/", ...)
        :return:  the official URI of the data collection
        
        ::

            >>> m.getOfficialDataTypeURI("chEBI")
            'urn:miriam:chebi'
        
        """
        res = self.serv.getOfficialDataTypeURI(nickname)
        return res

    def getResourceInfo(self, Id):
        """Retrieves the general information about a precise resource of a data collection.

        :param str Id: identifier of a resource (example: "MIR:00100005")
        :return: general information about a resource

        ::

            >>> m.getResourceInfo("MIR:00100005")
            'MIRIAM Resources (data collection)'


        """
        res = self.serv.getResourceInfo(Id)
        return res

    def getResourceInstitution(self, Id):
        """Retrieves the institution which manages a precise resource of a data collection.

        :param str Id: identifier of a resource (example: "MIR:00100005")
        :Returns: institution which manages a resource

        ::

            >>> m.getResourceInstitution("MIR:00100005")
            'European Bioinformatics Institute'


        """
        res = self.serv.getResourceInstitution(Id)
        return res

    def getResourceLocation(self, Id):
        """Retrieves the location of the servers of a location.

        :param str Id: identifier of a resource (example: "MIR:00100005")
        :return:  location of the servers of a resource

        ::

            >>> m.getResourceLocation("MIR:00100005")
            'United Kingdom'

        """
        res = self.serv.getResourceLocation(Id)
        return res

    def getServicesInfo(self):
        """Retrieves some information about these Web Services.

        :return: information about the Web Services

        ::

            >>> m.getServicesInfo()
        """
        res = self.serv.getServicesInfo()
        return res

    def getServicesVersion(self):
        """Retrieves the current version of MIRIAM Web Services.

        :return: Current version of the web services
        """
        return self.serv.getServicesVersion()

    def getURI(self, name, Id):
        """Retrieves the unique URI of a specific entity (example: "urn:miriam:obo.go:GO%3A0045202").

        If the data collection does not exist (or is not recognised), an empty String is returned.

        If the identifier is invalid for the given data collection, 'null' is returned.

        :param str name: name of a data collection (examples: "ChEBI", "UniProt")
        :param str id: identifier of an entity within the data collection (examples: "GO:0045202", "P62158")

        ::

            >>> m.getURI("UniProt", "P62158")
            'urn:miriam:uniprot:P62158'

        """
        res = self.serv.getURI(name, Id)
        return res

    def getURIs(self, names, Ids):
        """Retrieves the unique URIs for a list of specific entities (example: "urn:miriam:obo.go:GO%3A0045202").

        If a data collection does not exist (or is not recognised), an empty String is returned for this data collection.

        If an identifier is invalid for the given data collection, 'null' is returned for this data collection.

        If the provided lists do not have the same size, 'null' is returned.

        If the size of any list exceeds 200, 'null' is returned.

        :return: list of MIRIAM URIs

        ::

            >>> m.serv.getURIs(["UniProt", "GO"], ["P62158", "GO:0045202"])
            ['urn:miriam:uniprot:P62158', 'urn:miriam:obo.go:GO%3A0045202']

        """
        res = self.serv.getURIs(names, Ids)
        return res

    def isDeprecated(self, uri):
        """Says if a URI of a data collection is deprecated.

        :return: answer ("true" or "false") to the question: is this URI deprecated?

        ::

            >>> im.isDeprecated("urn:miriam:uniprot")
            False
        """
        res = self.serv.isDeprecated(uri)
        res = self._boolean_convertor(res)
        return res
    






from bioservices import BioMart
import unittest


def test_general():
    s = BioMart()
    #s.registry()
    s.datasets("ensembl")
    s.version("ensembl")
    s.attributes("oanatinus_gene_ensembl")
    s.filters("oanatinus_gene_ensembl")
    s.configuration("oanatinus_gene_ensembl")

    xmlq = """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE Query>
<Query  virtualSchemaName = "default" formatter = "TSV" header = "0" uniqueRows = "0" count = "" datasetConfigVersion = "0.6" >
                        
        <Dataset name = "pathway" interface = "default" >
                <Filter name = "referencepeptidesequence_uniprot_id_list" value = "P43403"/>
                <Attribute name = "stableidentifier_identifier" />
                <Attribute name = "pathway_db_id" />
        </Dataset>
</Query>
"""
    s.query(s._xml_example)


    # build own xml using the proper functions
    s.add_dataset_to_xml("protein")
    s.get_xml()

def _test_reactome_example():
    # this is not working anymore...
    s = biomart.BioMart()
    s.lookfor("reactome")
    s.datasets("REACTOME")
    #['interaction', 'complex', 'reaction', 'pathway']
    s.new_query()
    s.add_dataset_to_xml("pathway")
    s.add_filter_to_xml("species_selection", "Homo sapiens")
    s.add_attribute_to_xml("pathway_db_id")
    s.add_attribute_to_xml("_displayname")
    xmlq = s.get_xml()
    res = s.query(xmlq)




#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s):
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: unichem.py 275 2013-12-06 10:24:09Z cokelaer $

"""This module provides a class :class:`UniChem` 

.. topic:: What is UniChem

    :URL:  https://www.ebi.ac.uk/unichem/info/webservices
    :REST:  https://www.ebi.ac.uk/unichem/rest


    .. highlights::

        "UniChem is a 'Unified Chemical Identifier' system, designed to assist
        in the rapid cross-referencing of chemical structures, and their identifiers,
        between databases (read more). "

        -- From UniChem web page June 2013


"""






from bioservices import RESTService




class UniChem(RESTService):

    _url = "http://www.ebi.ac.uk/unichem/rest"
    def __init__(self, verbose=True):
        """**Constructor** `UniChem <https://www.ebi.ac.uk/unichem/>`_

        :param verbose: set to False to prevent informative messages
        """
        super(UniChem, self).__init__(name="UniChem", url=UniChem._url, verbose=verbose)
        self.source_ids = {
            "chembl":1, 
            "drugbank":2,
            "pdb":3,
            "iuphar":4,
            "pubchem_dotf":5,
            "kegg_ligand":6,
            "chebi":7,
            "nih_ncc":8,
            "zinc":9,
            "emolecules":10,
            "ibm":11,
            "atlas":12,
            "patents":13,
            "fdasrc":14,
            "surechem":15,
            "pharmgkb":17
            }# there was no 16 when I looked at the web site June 2013

    def get_source_id(self, src_id):
        """Returns the source id given a source name or source id

        Check that the source id is correct by looking at the :attr:`source_ids`
        attribute.

        """
        if src_id in self.source_ids.keys():
            src_id = self.source_ids[src_id]
        elif isinstance(src_id, int) and src_id in self.source_ids.values():
            pass
        elif isinstance(src_id, str) and src_id.isdigit():
            if int(src_id) in self.source_ids.values():
                src_id = int(src_id)
            else:
                raise ValueError("unrecognised src_id parameter. Valid names are %s"
                    % self.source_ids.keys())
        else:
            raise ValueError("unrecognised src_id parameter. Valid names are %s"
                % self.source_ids.keys())

        return src_id

    def get_src_compound_ids_from_src_compound_id(self, src_compound_id, src_id, target=None):
        """Obtain a list of all src_compound_ids from all sources which are
        CURRENTLY assigned to the same structure as a currently assigned query
        src_compound_id. 

        The output will include query src_compound_id if it is a valid
        src_compound_id with a current assignment. Note also, that by adding an
        additional (optional) argument (a valid src_id), then results will be restricted
        to only the source specified with this optional argument.

        :param str src_compound_id: a valid compound identifier
        :param str src_id: one of the valid database ids. See :attr:`source_ids`.
        :param str,int target: database identifier (name or id) to map to.

        :return: list of two element arrays, containing 'src_compound_id' and 'src_id',
            or (if optional 'to_src_id' is specified) list of 'src_compound_id's.

        ::

            >>> get_src_compound_ids_from_src_compound_id("CHEMBL12", "chembl")
            >>> get_src_compound_ids_from_src_compound_id("CHEMBL12", "chembl", "chebi")

        """
        src_id = self.get_source_id(src_id)
        query = "src_compound_id/%s/%s" % (src_compound_id, src_id)
        if target:
            target = self.get_source_id(target)
            query += "/%s" % target

        res = self.request(query)

        # the json string returned can be evaluated in Python.
        res = eval(res)
        return res

    def get_src_compound_ids_all_from_src_compound_id(self, src_compound_id,
        src_id, target=None):
        """Obtain a list of all src_compound_ids from all sources (including
        BOTH current AND obsolete assignments) to the same structure as a currently
        assigned query src_compound_id. 

        The output will include query src_compound_id if
        it is a valid src_compound_id with a current assignment. Note also, that by
        adding an additional (optional) argument (a valid src_id), then results will be
        restricted to only the source specified with this optional argument.

        :param str src_compound_id: a valid compound identifier
        :param source: one of the valid database ids. See :attr:`source_ids`.
        :param target: return answer for a specific target database only. Valid
            database ids can be found in :attr:`source_ids`.
        :return: list of three element arrays, containing 'src_compound_id' and 'src_id',
            and 'Assignment', or (if optional 'to_src_id' is specified) list of two element
            arrays, containing 'src_compound_id' and 'Assignment'.

        ::

            >>> get_src_compound_ids_all_from_src_compound_id("CHEMBL12", "chembl")
            >>> get_src_compound_ids_all_from_src_compound_id("CHEMBL12", "chembl", "chebi")

        """
        src_id = self.get_source_id(src_id)
        query = "src_compound_id_all/%s/%s" % (src_compound_id, src_id)
        if target:
            query += "/%s" % self.source_ids[target]

        res = self.request(query)

        # the json string returned can be evaluated in Python.
        res = eval(res)
        return res

    def get_mapping(self, source, target):
        """Obtain a full mapping between two sources. Uses only currently
        assigned src_compound_ids from both sources.

        :param source: name of the source database
        :param target: name of the target database
        :return: a dictionary. Keys are the source identifiers. Values are the
            target identifiers.

        ::

            >>> get_mapping("kegg_ligand", "chembl")

        """
        self.checkParam(source, self.source_ids.keys())
        self.checkParam(target, self.source_ids.keys())

        query = "mapping/%s/%s/" % (self.source_ids[source], self.source_ids[target])
        res = self.request(query)
        # evaluation the string as a list
        res = eval(res)
        # convert to a convenient dictionary
        mapping = [(x[str(self.source_ids[source])], x[str(self.source_ids[target])]) for x in res]
        mapping = dict(mapping)
        return mapping

    def get_src_compound_ids_from_inchikey(self, inchikey):
        """Obtain a list of all src_compound_ids (from all sources) which are
            CURRENTLY assigned to a query InChIKey

        :param str inchikey: input source identified by its InChiKey
        :return: list of two element arrays, containing 'src_compound_id' and 'src_id'.

        ::

            >>> uni.get_src_compound_ids_from_inchikey("AAOVKJBEBIDNHE-UHFFFAOYSA-N")
        """
        query = "inchikey/%s" % inchikey
        res = self.request(query)
        res = eval(res)
        return res

    def get_src_compound_ids_all_from_inchikey(self, inchikey):
        """Description:  Obtain a list of all src_compound_ids (from all sources) which
        have current AND obsolete assignments to a query InChIKey

        :param str inchikey: input source identified by its InChiKey
        :return: list of two element arrays, containing 'src_compound_id' and 'src_id'.
            and 'Assignment'.

        ::
    
            >>> uni.get_src_compound_ids_all_from_inchikey("AAOVKJBEBIDNHE-UHFFFAOYSA-N")

        """
        query = "inchikey_all/%s" % inchikey
        res = self.request(query)
        res = eval(res)
        return res

    def get_all_src_ids(self):
        """Obtain all src_ids of database currently in UniChem

        :return: list of 'src_id's.

        ::

            >>> uni.get_all_src_ids()

        """
        res = self.request("src_ids")
        res = eval(res)
        return res


    def get_source_information(self, src_id):
        """Description:  Obtain all information on a source by querying with a source id

        :param int src_id: identifier of a source database. 
        :return: dictionary containing:

            * src_id (the src_id for this source),
            * src_url (the main home page of the source),
            * name (the unique name for the source in UniChem, always lower case),
            * name_long (the full name of the source, as defined by the source),
            * name_label (A name for the source suitable for use as a 'label' for the source
                within a web-page. Correct case setting for source, and always less than 30
                characters),
            * description (a description of the content of the source),
            * base_id_url_available (an flag indicating whether this source provides a valid
            * base_id_url for creating cpd-specific links [1=yes, 0=no]).
            * base_id_url (the base url for constructing hyperlinks to this source [append an
            * identifier f    rom this source to the end of this url to create a valid url to a
            * specific page for this cpd], unless aux_for_url=1),
            * aux_for_url (A flag to indicate whether the aux_src field should be used to create 
                hyperlinks instead of the src_compound_id [1=yes, 0=no]

        Example:  https://www.ebi.ac.uk/unichem/rest/sources/1"""

        src_id = self.get_source_id(src_id)
        query = "sources/%s" % src_id
        res = self.request(query)
        res = eval(res)
        res = dict(res[0])
        return res

    def get_structure(self, src_compound_id, src_id):
        """Obtain structure(s) CURRENTLY assigned to a query src_compound_id.

        :param str src_compound_id: a valid compound identifier
        :param int src_id: corresponding database identifier (name or id). 

        :return:  dictionary with 'standardinchi' and 'standardinchikey' keys

        ::

            >>> uni.get_structure("CHEMBL12", "chembl")

        """
        src_id = self.get_source_id(src_id)
        query = "structure/%s/%s" % (src_compound_id, src_id)
        res = self.request(query)
        res = eval(res)
        res = res[0]
        return res

    def get_structure_all(self, src_compound_id, src_id):
        """Obtain structure(s) with current AND obsolete assignments

        :param str src_compound_id: a valid compound identifier
        :param int src_id: corresponding database identifier (name or id). 
        :return:  dictionary with 'standardinchi', 'standardinchikey' and 
            'assignment' keys

        ::

            >>> uni.get_structure_all("CHEMBL12", "chembl")

        """
        src_id = self.get_source_id(src_id)
        query = "structure_all/%s/%s" % (src_compound_id, src_id)
        res = self.request(query)
        res = eval(res)
        res = res[0]
        return res

    def get_src_compound_id_url(self, src_compound_id, src_id, to_src_id):
        """Obtain a list of URLs for all src_compound_ids

        Obtain a list of URLs for all src_compound_ids
        from a specifiedsource (the 'to_src_id'), which are CURRENTLY assigned to the same structure as
        a currently assigned query src_compound_id. Method only applicable for sources
        which support direct URLs to src_compound_id pages. Method also applicable for
        'to_src_id's where the hyperlink is constructed from auxiliary data [and not
        from the src_compound_id] as per example2 below.

        :param str src_compound_id: a valid compound identifier
        :param int src_id: corresponding database identifier (name or id).
        :param int to_src_id: database identifier (name or id) to map to.
        :param str to_src_id:
        :return: list of URLs.

        ::

            >>> uni.get_src_compound_id_url("CHEMBL12", "chembl", "drugbank")
            >>> # equivalent to 
            >>> uni.get_src_compound_id_url("CHEMBL12", 1, 2)


        """
        src_id = self.get_source_id(src_id)
        to_src_id = self.get_source_id(to_src_id)
        query = "src_compound_id_url/%s/%s/%s" % (src_compound_id, src_id, to_src_id)
        res = self.request(query)
        res = eval(res)
        res = [x['url'] for x in res]
        return res

    def get_src_compound_ids_all_from_obsolete(self, obsolete_src_compound_id,
        src_id, to_src_id=None):
        """Obtain a list of all src_compound_ids from all sources with BOTH
        current AND obsolete to the same structure with an obsolete assignment to the 
        query src_compound_id. 

        The output will include query src_compound_id if it is a
        valid src_compound_id with an obsolete assignment. Note also, that by adding an
        additional (optional) argument (a valid src_id), then results will be restricted
        to only the source specified with this optional argument.

        :param str src_compound_id: a valid compound identifier
        :param int src_id: corresponding database identifier (name or id).
        :param int to_src_id: database identifier (name or id) to map to.
        :return: list of four element arrays, containing 'src_compound_id', 'src_id',
            'assignment' and 'UCI', or (if optional 'to_src_id' is specified) list of
            three element arrays, containing 'src_compound_id', 'Assignment' and 'UCI'.


        ::

            >>> get_src_compound_ids_all_from_obsolete("DB07699", "2")
            >>> get_src_compound_ids_all_from_obsolete("DB07699", "2", "1")
        """
        src_id = self.get_source_id(src_id)
        query = "src_compound_id_all_obsolete/%s/%s" % (obsolete_src_compound_id, src_id)
        if to_src_id:
            to_src_id = self.get_source_id(to_src_id)
            query += "/%s" % to_src_id

        res = self.request(query)

        # the json string returned can be evaluated in Python.
        res = eval(res)
        return res


    def get_verbose_src_compound_ids_from_inchikey(self, inchikey):
        """Description:  Obtain all src_compound_ids (from all sources) which are CURRENTLY
        assigned to a query InChIKey. However, these are returned as part of the
        following data structure: A list of sources containing these src_compound_ids,
        including source description, base_id_url, etc. One element in this list is a
        list of the src_compound_ids currently assigned to the query InChIKey.
    
        :param str inchikey: input source identified by its InChiKey
        :return: list containing

            * src_id (the src_id for this source),
            * src_url (the main home page of the source),
            * name (the unique name for the source in UniChem, always lower case),
            * name_long (the full name of the source, as defined by the source),
            * name_label (A name for the source suitable for use as a 'label' for the source
                within a web-page. Correct case setting for source, and always less than 30
                characters),
            * description (a description of the content of the source),
            * base_id_url_available (an flag indicating whether this source provides a valid
            * base_id_url for creating cpd-specific links [1=yes, 0=no]).
            * base_id_url (the base url for constructing hyperlinks to this source [append an
            * identifier from this source to the end of this url to create a valid url to a
                specific pag    e for this cpd], unless aux_for_url=1),
            * aux_for_url (A flag to indicate whether the aux_src field should be used to
                create hyperlinks instead of the src_compound_id [1=yes, 0=no] ,
            * src_compound_id (a list of src_compound_ids from this source which are currently
                assigned to the query InChIKey.
            * aux_src (a list of src-compound_id keys mapping to corresponding auxiliary data
                (url_id:value), for creating links if aux_for_url=1. Only shown if
                aux_for_url=1).

        ::

            >>> uni.get_verbose_src_compound_ids_from_inchikey("ZUITABIAKMVPG-UHFFFAOYSA-N")

        """
        query = "verbose_inchikey/%s" % inchikey
        res = self.request(query)
        res = eval(res)
        return res

    def get_auxiliary_mappings(self, src_id):
        """For a single source, obtain a mapping between all current
            src_compound_ids to their corresponding auxiliary data if any.

        Some instances of UniChem may contain sources that create URLs
        for compound-specific pages by using strings or identifiers (called 'auxiliary
        data' here) that are different to the src_compound_ids for the source. This is
        not very common, but is dealt with in UniChem by use of an additional mapping
        step for these sources. This function returns such mapping. 

        .. warning:: this method may return very large data sets, and may therefore take a
            very long time to retrieve. A much faster method of retrieving this same data
            set is to download the pre-cached, gzipped mapping file for the source of
            interest from the `Auxiliary Data Mapping page <https://www.ebi.ac.uk/unichem/wholesourcemap>`_

        :param int src_id: corresponding database identifier (name or id).
        :return: list of two element arrays, containing 'src_compound_id' and 'auxiliary data'.

        ::
        
            >>> uni.get_auxiliary_mappings(15)

        """

        src_id = self.get_source_id(src_id)
        info = self.get_source_information(src_id)
        if info['aux_for_url'] == '1':
            query = "mappingaux/%s" % src_id
            res = self.request(query)
            #res = eval(res)
            return res
        else:
            self.logging.warning("This function accepts src_id that have auxiliary data only")




import __main__
__main__.pymol_argv = [ 'pymol', '-qc'] # Quiet and no GUI

import os
if os.path.isfile("bioservices_pdb.png"):
    os.remove("bioservices_pdb.png")

# BioServices 1: obtain the PDB ID from a given uniprot ID (P43403 i.e. ZAP70)
from bioservices import *
print("Retrieving PDB ID")
u = UniProt(verbose=False)
res = u.mapping(fr="ID", to="PDB_ID", query="P43403", format="tab")
pdb_id = res[3]   # "1FBV"

# BioServices 2: Download the PDB file from the PDB Web Service
print("Fetching PDB file")
p = pdb.PDB()
res = p.getFile(pdb_id, "pdb")

# General: save the fetched file in a temporary file
import tempfile
fh = tempfile.NamedTemporaryFile()
fh.write(res)
sname = fh.name

# THIS IS NOT BIOSERVICES ANYMORE but PYMOL 
import pymol
pymol.finish_launching()
pymol.cmd.load(sname)
pymol.cmd.png("bioservices_pdb.png", width="15cm", height="15cm", dpi=140)
#pymol.cmd.png("my_image.png")
# Get out!
pymol.cmd.quit()



# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: hgnc.py 291 2013-12-10 18:14:19Z cokelaer $
"""Interface to HUGO/HGNC web services

.. topic:: What is HGNC ?

    :URL: http://www.genenames.org
    :Citation:

    .. highlights::

        "The HUGO Gene Nomenclature Committee (HGNC) has assigned unique gene symbols and
        names to over 37,000 human loci, of which around 19,000 are protein coding.
        genenames.org is a curated online repository of HGNC-approved gene nomenclature
        and associated resources including links to genomic, proteomic and phenotypic
        information, as well as dedicated gene family pages."

        -- From HGNC web site, July 2013


"""
from bioservices import RESTService, xmltools
from xmltools import bs4
from urllib2 import HTTPError


__all__ = ["HGNC"]


class HGNC(RESTService):
    """Interface to the `HGNC <http://www.genenames.org>`_ service


    ::

        >>> from bioservices import *
        >>> # Fetch XML document for gene ZAP70
        >>> s = HGNC()
        >>> xml = s.get_xml("ZAP70")
        >>> # You can fetch several gene names:
        >>> xml = s.get_xml("ZAP70;INSR")
        >>> # Wrong gene name request returns an empty list
        >>> s.get_xml("wrong")
        []

    For a single name, the following methods are available::

        >>> # get the aliases of a given gene
        >>> print(s.get_aliases("ZAP70"))
        [u'ZAP-70', u'STD']
        >>> # get UniProt accession code
        >>> s.get_xrefs("ZAP70")['UniProt']['xkey']
        'P43403'
        >>> # get XML link to a UniProt cross-reference
        >>> s.get_xrefs("ZAP70", "xml")['UniProt']['link']
        ['http://www.uniprot.org/uniprot/P43403.xml']


    You can access to the links of a cross reference as well::

        values = s.get_xrefs("ZAP70")
        s.onWeb(values['EntrezGene']['link'][0])


    :references: http://www.avatar.se/HGNC/doc/tutorial.html

    """
    def __init__(self, verbose=False):
        url = "http://www.avatar.se/HGNC/wr/gene"
        super(RESTService,self).__init__("HGNC", url=url, verbose=verbose)
        self._always_return_list = False

        #: Force XML to be checked for unicode consistency see :class:`Service`
        self._fixing_unicode = True
        self._fixing_encoding = "utf-8"


    def _set_return(self, mode):
        assert mode in [False, True]
        self._always_return_list = mode
    def _get_return(self):
        return self.always_return_list

    def get_xml(self, gene):
        """Returns XML of a single gene or list of genes


        :param str gene: a valid gene name. Several gene names can be concatenated with
            comma ; character (e.g., 'ZAP70;INSR')


        .. doctest::

            >>> from bioservices import *
            >>> s = HGNC()
            >>> res = s.get_xml("ZAP70")
            >>> res.findAll("alias")
            >>> [x.text for x in res.findAll("alias")]
            [u'ZAP-70', u'STD']

        .. seealso:: :meth:`get_aliases`
        """
        try:
            if ";" in gene:
                res = self.request(self.url + "s/%s" % gene)
            else:
                res = self.request("%s.xml" % gene)
        except HTTPError:
            print("!!BioServices HTTPError caught in HGNC. Probably an invalid gene name")

            res = bs4.BeautifulSoup()
        #except Exception:
        #    raise Exception
        return res

    def _get_attribute(self, gene, attribute):
        res = self.get_xml(gene)
        values = [x.text.strip() for x in res.findAll(attribute)][0]
        if len(values) == 1:
            return values[0]
        else:
            return values

    def get_aliases(self, gene):
        """Get aliases for a single gene name"""
        res = self.get_xml(gene)
        aliases = [x.text for x in res.findAll("alias")]
        return aliases

    def get_name(self, gene):
        """Get name for a single gene name"""
        return self._get_attribute(gene, "name")

    def get_chromosome(self, gene):
        """Get chromosome for a single gene name"""
        return self._get_attribute(gene, "chromosome")

    def get_previous_symbols(self, gene):
        """Get previous symbols for a single gene name"""
        return self._get_attribute(gene, "previous_symbols")

    def get_withdrawn_symbols(self, gene):
        """Get withdrawn symbols for a single gene name"""
        return self._get_attribute(gene, "withdrawn_symbols")

    def get_previous_names(self, gene):
        """Get previous names for a single gene name"""
        return self._get_attribute(gene, "previous_names")

    def get_xrefs(self, gene, keep="html"):
        """Get the cross references for a given single gene name

        ::


            >>> databases = s.get_xrefs("ZAP70").keys()

            >>> # get XML link to a UniProt cross-reference
            >>> s.get_xrefs("ZAP70", "xml")['UniProt']['link']
            ['http://www.uniprot.org/uniprot/P43403.xml']


        """
        assert keep in ["html", "xml", "fasta", "rdf", "txt", "gff", None]
        # returns list of dictionary that contains the attributes of each
        # reference as well as a list of links provided for each reference.

        xml = self.get_xml(gene)
        values = self._get_xref(xml, keep)
        return values

    def _get_xref(self, xml, keep):
        # get all dbs and build up a dict out of it
        dbs =  [x.attrs['xdb'] for x in xml.findAll("xref")]
        values =  dict([(this,{}) for this in dbs])

        # rescan the xml to get the other attributes
        refs = [x.attrs for x in xml.findAll("xref")]
        for ref in refs:
            db = ref['xdb']
            values[db] = ref.copy()
            # this looks quite complicated so here is a bit of explanation:
            # Each reference may have a few links. However, we are interested
            # only in this function by the HTML format. So, for a given database (res.findAll(xref)),
            # we search for all links (findAll(link)) and for each link found, we keep only those where
            # format is HTML. Finally; we get only the attribute 'xlink:href'
            links = [y.attrs['xlink:href'] for y in [x for x in xml.findAll("xref") if x['xdb']==db][0].findAll("link") if y.attrs['format']==keep]
            values[db]['link'] = links[:]

        return values

    def lookfor(self, pattern):
        """Finds all genes that starts with a given pattern

        :param str pattern: a string. Could be the wild character `*`
        :return: list of dictionary. Each dictionary contains the 'acc',
            'xlink:href' and 'xlink:title' keys


        .. doctest::

            >>> from bioservices import *
            >>> s = HGNC()
            >>> s.lookfor("ZAP")
            [{'acc': 'HGNC:12858',
            'xlink:href': '/HGNC/wr/gene/ZAP70',
            'xlink:title': 'ZAP70'}]


        This function may be used to count the number of entries::


            len(s.lookfor('*'))

        """
        params = {'search': 'symbol', 'value':pattern}
        # note the extra s before ;index.xml
        xml = self.request(self.url + "s;index.xml?" + self.urlencode(params))

        res = [x.attrs for x in xml.findAll("gene")]
        return res

    def get_all_names(self):
        """Returns all gene names"""
        entries = self.lookfor("*")
        names = [entry['xlink:title'] for entry in entries]
        return names

    def mapping(self, value):
        """maps an identifier from a database onto HGNC database

        :param str value: a valid DB:id string (e.g. "UniProt:P36888")
        :return: a list of dictionary with the keys 'acc', 'xlink:href', 'xlink:title'

            >>> value = "UniProt:P43403"
            >>> res = s.mapping(value)
            >>> res[0]['xlink:title']
            'ZAP70'
            >>> res[0]['acc']
            'HGNC:12858'


        .. seealso:: :meth:`mapping_all`
        """
        
        xml = self.request(self.url + "s;index.xml?" + self.urlencode({'search': 'xref', 'value':value}))
        genes = xml.findAll("gene")
        res = [g.attrs for g in genes]
        return res

    def mapping_all(self, entries=None):
        """Retrieves cross references for more than one entry

        :param entries: list of values entries (e.g., returned by the :meth:`lookfor` method.)
            if not provided, this method looks for all entries.
        :returns: list of dictionaries with keys being all entry names. Values is a
		dictionary of cross references.

        .. warning:: takes 10 minutes

        """
        from math import ceil
        results = {}

        if entries==None:
            print("First, get all entries")
            entries = self.lookfor('*')

        names = [entry['xlink:title'] for entry in entries]
        N = len(names)

        # split query in sets of 100 names 

        dn = 300
        N = len(names)
        n = int(ceil(N/float(dn)))
        for i in range(0,n):
            print("Completed ", i+1 , "/", n)
            query  = ";".join(names[i*dn:(i+1)*dn])
            xml = self.get_xml(query)
            genes = xml.findAll("gene")
            for gene in genes:
                res = self._get_xref(gene, None)
                #acc = gene.attrs['acc'] not needed. can be access from ['HGNC']['xkey']
                name = gene.attrs['symbol']
                results[name] = res.copy()
        return results











# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: uniprot.py 295 2013-12-10 21:29:39Z cokelaer $
"""Interface to some part of the UniProt web service

.. topic:: What is UniProt ?

    :URL: http://www.uniprot.org
    :Citation:

    .. highlights::

        "The Universal Protein Resource (UniProt) is a comprehensive resource for protein
        sequence and annotation data. The UniProt databases are the UniProt
        Knowledgebase (UniProtKB), the UniProt Reference Clusters (UniRef), and the
        UniProt Archive (UniParc). The UniProt Metagenomic and Environmental Sequences
        (UniMES) database is a repository specifically developed for metagenomic and
        environmental data."

        -- From Uniprot web site (help/about) , Dec 2012


.. mapping between uniprot and bunch of other DBs.
.. ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/idmapping/
.. http://www.uniprot.org/docs/speclist
.. http://www.uniprot.org/docs/pkinfam

"""
from services import Service, RESTService
import urllib2
import urllib

__all__ = ["UniProt"]


mapping = {"UniProtKB AC/ID":"ACC+ID", 
    "UniProtKB": "ACC",
    "UniProtKB": "ID",
    "UniParc": "UPARC",
    "UniRef50": "NF50",
    "UniRef90": "NF90",
    "UniRef100": "NF100",
    "EMBL/GenBank/DDBJ": "EMBL_ID",
    "EMBL/GenBank/DDBJ CDS": "EMBL",
    "PIR": "PIR",
    "UniGene": "UNIGENE_ID",
    "Entrez Gene (GeneID)": "P_ENTREZGENEID",
    "GI number*":"P_GI", 
    "IPI": "P_IPI",
    "RefSeq Protein": "P_REFSEQ_AC",
    "RefSeq Nucleotide": "REFSEQ_NT_ID",
    "PDB": "PDB_ID",
    "DisProt": "DISPROT_ID",
    "HSSP": "HSSP_ID",
    "DIP": "DIP_ID",
    "MINT": "MINT_ID",
    "Allergome": "ALLERGOME_ID",
    "MEROPS": "MEROPS_ID",
    "mycoCLAP": "MYCOCLAP_ID",
    "PeroxiBase": "PEROXIBASE_ID",
    "PptaseDB": "PPTASEDB_ID",
    "REBASE": "REBASE_ID",
    "TCDB": "TCDB_ID",
    "PhosSite": "PHOSSITE_ID",
    "DMDM": "DMDM_ID",
    "Aarhus/Ghent-2DPAGE": "AARHUS_GHENT_2DPAGE_ID",
    "World-2DPAGE": "WORLD_2DPAGE_ID",
    "DNASU": "DNASU_ID",
    "Ensembl": "ENSEMBL_ID",
    "Ensembl Protein": "ENSEMBL_PRO_ID",
    "Ensembl Transcript": "ENSEMBL_TRS_ID",
    "Ensembl Genomes": "ENSEMBLGENOME_ID",
    "Ensembl Genomes Protein": "ENSEMBLGENOME_PRO_ID",
    "Ensembl Genomes Transcript": "ENSEMBLGENOME_TRS_ID",
    "GeneID": "P_ENTREZGENEID",
    "GenomeReviews": "GENOMEREVIEWS_ID",
    "KEGG": "KEGG_ID",
    "PATRIC": "PATRIC_ID",
    "UCSC": "UCSC_ID",
    "VectorBase": "VECTORBASE_ID",
    "AGD": "AGD_ID",
    "ArachnoServer": "ARACHNOSERVER_ID",
    "CGD": "CGD",
    "ConoServer": "CONOSERVER_ID",
    "CYGD": "CYGD_ID",
    "dictyBase": "DICTYBASE_ID",
    "EchoBASE": "ECHOBASE_ID",
    "EcoGene": "ECOGENE_ID",
    "euHCVdb": "EUHCVDB_ID",
    "EuPathDB": "EUPATHDB_ID",
    "FlyBase": "FLYBASE_ID",
    "GeneCards": "GENECARDS_ID",
    "GeneFarm": "GENEFARM_ID",
    "GenoList": "GENOLIST_ID",
    "H-InvDB": "H_INVDB_ID",
    "HGNC": "HGNC_ID",
    "HPA": "HPA_ID",
    "LegioList": "LEGIOLIST_ID",
    "Leproma": "LEPROMA_ID",
    "MaizeGDB": "MAIZEGDB_ID",
    "MIM": "MIM_ID",
    "MGI": "MGI_ID",
    "neXtProt": "NEXTPROT_ID",
    "Orphanet": "ORPHANET_ID",
    "PharmGKB": "PHARMGKB_ID",
    "PomBase": "POMBASE_ID",
    "PseudoCAP": "PSEUDOCAP_ID",
    "RGD": "RGD_ID",
    "SGD": "SGD_ID",
    "TAIR": "TAIR_ID",
    "TubercuList": "TUBERCULIST_ID",
    "WormBase": "WORMBASE_ID",
    "WormBase Transcript": "WORMBASE_TRS_ID",
    "WormBase Protein": "WORMBASE_PRO_ID",
    "Xenbase": "XENBASE_ID",
    "ZFIN": "ZFIN_ID",
    "eggNOG": "EGGNOG_ID",
    "GeneTree": "GENETREE_ID",
    "HOGENOM": "HOGENOM_ID",
    "HOVERGEN": "HOVERGEN_ID",
    "KO": "KO_ID",
    "OMA": "OMA_ID",
    "OrthoDB": "ORTHODB_ID",
    "ProtClustDB": "PROTCLUSTDB_ID",
    "BioCyc": "BIOCYC_ID",
    "Reactome": "REACTOME_ID",
    "UniPathWay": "UNIPATHWAY_ID",
    "CleanEx": "CLEANEX_ID",
    "GermOnline": "GERMONLINE_ID",
    "ChEMBL": "CHEMBL_ID",
    "ChiTaRS": "CHITARS_ID",
    "DrugBank": "DRUGBANK_ID",
    "GenomeRNAi": "GENOMERNAI_ID",
    "NextBio": "NEXTBIO_ID"
}




class UniProt(RESTService):
    """Interface to the `UniProt <http://www.uniprot.org>`_ service

    .. rubric:: Identifiers mapping between databases:

    ::

        >>> from bioservices import UniProt
        >>> u = UniProt(verbose=False)
        >>> u.mapping("ACC", "KEGG_ID", query='P43403')
        defaultdict(<type 'list'>, {'P43403': ['hsa:7535']})
        >>> res = u.search("P43403")

        # Returns sequence on the ZAP70_HUMAN accession Id
        >>> sequence = u.search("ZAP70_HUMAN", columns="sequence")

    """
    _mapping = mapping.copy()
    _url = "http://www.uniprot.org"
    def __init__(self, verbose=True):
        """**Constructor**

        :param verbose: set to False to prevent informative messages
        """
        super(UniProt, self).__init__(name="UniProt", url=UniProt._url, verbose=verbose)


    def mapping(self, fr="ID", to="KEGG_ID",  query="P13368", format="tab"):
        """This is an interface to the UniProt mapping service


        :param fr: the source database identifier. See :attr:`_mapping`.
        :param to: the targetted database identifier. See :attr:`_mapping`.
        :param query: a string containing one or more IDs separated by a space
            It can also be a list of strings.
        :param format: the output format (default is tabulated "tab")
            only tab should be used for now.
        :return: a list. The first element is the source database Id. The second
            is the targetted source identifier. Following elements are alternate
            of one the entry and its mapped Id. If a query has several mapped
            Ids, the query is repeated (see example with PDB mapping here below)
            e.g., ["From:ID", "to:PDB_ID", "P43403"]

        ::

            >>> u.mapping("ACC", "KEGG_ID", 'P43403')
            defaultdict(<type 'list'>, {'P43403': ['hsa:7535']})
            >>> u.mapping("ACC", "KEGG_ID", 'P43403 P00958')
            defaultdict(<type 'list'>, {'P00958': ['sce:YGR264C'], 'P43403': ['hsa:7535']})
            >>> u.mapping("ID", "PDB_ID", "P43403", format="tab")
            defaultdict(<type 'list'>, {'P43403': ['1FBV', '1M61', '1U59',
            '2CBL', '2OQ1', '2OZO', '2Y1N', '3ZNI', '4A4B', '4A4C', '4K2R']})



        There is a web page that gives the list of correct `database identifiers
        <http://www.uniprot.org/faq/28>`_. You can also look at the
        :attr:`_mapping` attribute.

        :URL: http://www.uniprot.org/mapping/


        .. versionchanged:: 1.1.1 to return a dictionary instaed of a list
        .. versionchanged:: 1.1.2 the values for each key is now made of a list 
            instead of strings so as to store more than one values.
        .. versionchanged:: 1.2.0 input query can also be a list of strings
            instead of just a string

        """
        url = self.url + '/mapping/'

        if isinstance(query, list):
            query = " ".join(query)
        params = {'from':fr, 'to':to, 'format':format, 'query':query}
        params = self.urlencode(params)
        result = self._request_timeout(url+"?"+params, format=format)

        # let us improve the output a little bit using a list instead of a
        # string
        try:
            result = result.split()
            result[0]+=':'+fr
            result[1]+=':'+to
        except:
            pass

	    # changes in version 1.1.1 returns a dictionary instead of list
        del result[0]
        del result[0]
        if len(result) == 0:
            return {}
        else:
            # bug fix based on ticket #19 version 1.1.2
            # the default dict set empty list for all keys by default
            from collections import defaultdict
            result_dict = defaultdict(list)

            keys = result[0::2]
            values = result[1::2]
            for i, key in enumerate(keys):
                result_dict[key].append(values[i])
        return result_dict

    def searchUniProtId(self, uniprot_id, format="xml"):
        """Search for a uniprot ID in UniprotKB database

        :param str uniprot: a valid uniprotKB ID
        :param str format: expected output format amongst xml, txt, fasta, gff, rdf

        ::

            >>> u = UniProt()
            >>> res = u.searchUniProtId("P09958", format="xml")


        """
        _valid_formats = ['txt', 'xml', 'rdf', 'gff', 'fasta']
        self.checkParam(format, _valid_formats)
        #if format not in _valid_formats:
        #    raise ValueError("invalid format provided. Use one of %s" % _valid_formats)
        url = self.url + "/uniprot/" + uniprot_id + '.' + format
        res = self.request(url, format=format)
        return res

    def get_fasta(self, id_):
        """Returns FASTA string given a valid identifier
        
        
        .. seealso:: :mod:`bioservices.apps.fasta` for dedicated tools to
            manipulate FASTA
        """
        from bioservices.apps.fasta import FASTA
        f = FASTA()
        f.get_fasta(id_)
        return f.fasta
        

    def get_fasta_sequence(self, id_):
        """Returns FASTA sequence (Not FASTA)

        :param str id_: Should be the entry name
        :return: returns fasta sequence (string)

        .. warning:: this is the sequence found in a fasta file, not the fasta
            content itself. The difference is that the header is removed and the
            formatting of end of lines every 60 characters is removed.

        """
        from bioservices.apps.fasta import FASTA
        f = FASTA()
        f.load_fasta(id_)
        return f.sequence


    def search(self, query, format="tab", columns=None,
        include=False,sort="score", compress=False, limit=None, offset=None, maxTrials=10):
        """Provide some interface to the uniprot search interface.

        :param str query: query must be a valid uniprot query.
            See http://www.uniprot.org/help/text-search, http://www.uniprot.org/help/query-fields 
        :param str format: a valid format amongst html, tab, xls, asta, gff,
            txt, xml, rdf, list, rss. If tab or xls, you can also provide the 
            columns argument.  (default is tab)
        :param str columns: comma-separated list of values. Works only if fomat 
            is tab or xlsFor UnitProtKB, the possible columns are:
            citation, clusters, comments, database, domains, domain, ec, id, entry name
            existence, families, features, genes, go, go-id, interpro, interactor,
            keywords, keyword-id, last-modified, length, organism, organism-id, 
            pathway, protein names, reviewed, score, sequence, 3d, 
            subcellular locations, taxon, tools, version, virus hosts. The
            column database must be follows by the database name in brackets
            (e.g. "database(PDB)")
        :param bool include:  include isoform sequences when the format
            parameter is fasta. Include description when format is rdf. 
        :param sort: by score by default. Set to None to bypass this behaviour
        :param bool compress: gzip the results
        :param int limit: Maximum number of results to retrieve.
        :param int offset:  Offset of the first result, typically used together
            with the limit parameter.
        :param int maxTrials: this request is unstable, so we may want to try
            several time.

        To obtain the list of uniprot ID returned by the search of zap70 can be retrieved as follows
        ::

            >>> u.search('zap70+AND+organism:9606', format='list')
            >>> u.search("zap70+and+taxonomy:9606", format="tab", limit=3, 
            ...    columns="entry name,length,id, genes")
            Entry name  Length  Entry   Gene names
            CBLB_HUMAN  982 Q13191  CBLB RNF56 Nbla00127
            CBL_HUMAN   906 P22681  CBL CBL2 RNF55
            CD3Z_HUMAN  164 P20963  CD247 CD3Z T3Z TCRZ

        other examples::

            u.search("ZAP70+AND+organism:9606", limit=3, columns="id,database(PDB)")


        .. warning:: this function request seems a bit unstable (UniProt web issue ?)
            so we repeat the request if it fails
        """
        params = {}

        if format!=None:
            _valid_formats = ['tab', 'xls', 'fasta', 'gff', 'txt', 'xml', 'rss', 'list', 'rss', 'html']
            self.checkParam(format, _valid_formats)
            params['format'] = format

        if columns!=None:
            self.checkParam(format, ["tab","xls"])
            _valid_columns = ['citation', 'clusters', 'comments','database',
                'domains','domain', 'ec','id','entry name','existence',
        		'families', 'features', 'genes', 'go', 'go-id', 'interpro', 
                'interactor', 'keywords', 'keyword-id', 'last-modified', 
                'length', 'organism', 'organism-id', 'pathway', 'protein names', 
                'reviewed', 'score', 'sequence', '3d', 'subcellular locations', 
                'taxonomy', 'tools', 'version', 'virus hosts']
            # remove unneeded spaces before/after commas if any
            if "," in columns:
                columns = [x.strip() for x in columns.split(",")]
            else:
                columns = [columns]

            for col in columns:
                if col.startswith("database(") == True:
                    pass
                else:
                    self.checkParam(col, _valid_columns)

            # convert back to a string as expected by uniprot
            params['columns'] = ",".join([x.strip() for x in columns])

        if include == True and format in ["fasta", "rdf"]:
            params['include'] = 'yes'

        if compress == True:
            params['compress'] = 'yes'
 
        if sort:
            self.checkParam(sort, ["score"])
            params['sort'] = sort

        if offset != None:
            if isinstance(offset, int):
                params['offset'] = offset

        if limit != None:
            if isinstance(limit, int):
                params['limit'] = limit

        params = self.urlencode(params)
        
        #res = s.request("/uniprot/?query=zap70+AND+organism:9606&format=xml", params)
        trials = 1
        while trials<maxTrials:
            try:
                res = self.request("uniprot/?query=%s" % query + "&" + params, "txt")
                trials = maxTrials + 1
            except:
                self.logging.warning("Trying again...")
                import time
                time.sleep(2)
                trials += 1
        return res



    def quick_search(self, query, include=False,sort="score", limit=None):

        res = self.search(query, "tab", include=include, sort=sort, limit=limit)

        #if empty result, nothing to do
        if len(res) == 0:
            return res
        # else populate a dictionary
        newres = {}
        for line in res.split("\n")[1:-1]:
            print line
            Entry, a,b,c,d,e,f = line.split("\t")
            print Entry, a, b, c, d, e, f
            newres[Entry] = { 'Entry name': a,
                         'Status': b,
                         'Protein names': c,
                         'Gene names': d,
                         'Organism' : e,
                         'Length' : f}
        return newres







# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: uniprot.py 226 2013-07-08 07:05:14Z cokelaer $
"""Interface to some part of the UniProt web service

.. topic:: What is UniProt ?

    :URL: http://www.uniprot.org
    :Citation:

    .. highlights::

        "The Universal Protein Resource (UniProt) is a comprehensive resource for protein
        sequence and annotation data. The UniProt databases are the UniProt
        Knowledgebase (UniProtKB), the UniProt Reference Clusters (UniRef), and the
        UniProt Archive (UniParc). The UniProt Metagenomic and Environmental Sequences
        (UniMES) database is a repository specifically developed for metagenomic and
        environmental data."

        -- From Uniprot web site (help/about) , Dec 2012


.. mapping between uniprot and bunch of other DBs.
.. ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/idmapping/
.. http://www.uniprot.org/docs/speclist
.. http://www.uniprot.org/docs/pkinfam

"""
from services import RESTService
from xmltools import readXML
__all__ = ["BioCarta"]


class BioCarta(RESTService):
    """Interface to `BioCarta <http://www.biocarta.com>`_ pages


    This is not a REST interface actually but rather a parser to some of the
    HTML pages relatd to pathways.

    One can retrieve the pathways names and their list of proteins. 

        >>> from bioservics import *
        >>> b = BioCarta()
        >>> pathways = b.get_pathway_names()
        >>> proteins = b.get_pathway_protein_names(pathways[0])


    .. warning:: biocarta pathways layout can be accesses from PID

    """
    _url = "http://www.biocarta.com/"
    def __init__(self, verbose=True):
        """**Constructor**

        :param verbose: set to False to prevent informative messages
        """
        super(BioCarta, self).__init__(name="BioCarta", url=BioCarta._url, verbose=verbose)
        self.fname  = "biocarta_pathways.txt"

        self._allPathwaysURL =  "http://www.biocarta.com/genes/allPathways.asp"

    def get_pathway_names(self, startswith=""):
        """returns pathways from biocarta
        
        all human and mouse. can perform a selectiom
        """
        x = readXML(self._allPathwaysURL)
        pathways = [this.get("href") for this in x.findAll("a") if "pathfiles" in this.get("href")]
        pathways =  [str(x.split("/")[-1]) for x in pathways] # split the drive
        pathways = sorted(list(set(pathways)))
        pathways = [x for x in pathways if x.startswith(startswith)]
        return pathways


    def get_pathway_protein_names(self, pathway):
        """returns list of list. Each elements is made of 3 items: gene name,
        locusId and accession (often empty
        
        Requires to parse HTML page such as 
        http://www.biocarta.com/pathfiles/m_actinYPathway.asp

        to figure out the URL that would pop up if we press the protein list
        button. For instance:

        http://www.biocarta.com/pathfiles/PathwayProteinList.asp?showPFID=175

        but now we need to parse the HTML, which is not necessaray very robust.
        THere are many tables and we want to access one that is a children of
        another... Finally, We scan the table for tr and td tags.

        The most difficult is to find the good table which is hardcoded to be
        the third that contains a th[0] == "Gene name". Although there is only
        one, 3 are returned due probably to an error in the parsing or the HTMl
        file itself. To be checked and made more robust.

        """
        url = self._url + "/pathfiles/" + pathway
        x = readXML(url)
        self.logging.info("Reading " + url)
        protein_url = [this.get("href") for this in x.findAll("a") if this.has_key("href") and "Protein" in this.get("href")]
        if len(protein_url) == 0:
            return None
        else:
            link = protein_url[0]
            link = link.split("/pathfiles/")[-1]
            link = str(link) # get rid of unicode ?
            link = link.strip("')")
            url = self._url + "/pathfiles/" + link
            self.logging.info("Reading " + url)
            x = readXML(url)

            # seems to work
            table = [this for this in x.findAll("table") if this.findAll("th")
                    and  this.findAll("th")[0].getText() == "Gene Name"][2]
            # now get the genename, locus and accession
            rows = [[y.getText() for y in x.findAll("td")] for x in  table.findAll("tr")]
            rows = [x for x in rows if len(x)]
            return rows



#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EMBL-EBI
#
#  File author(s):
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: xmltools.py 236 2013-09-02 17:58:04Z cokelaer $
"""This module includes common tools to manipulate XML files"""
from __future__ import print_function
import xml.etree.ElementTree as ET
import bs4
from bioservices import unicodefix 
import urllib2

__all__ = ["easyXML", "readXML"]




class easyXML(object):
    """class to ease the introspection of XML documents.

    This class uses the standard xml module as well as the package BeautifulSoup
    to help introspecting the XML documents.

    ::

        >>> from bioservices import *
        >>> n = ncbiblast.NCBIblast()
        >>> res = n.getParameters() # res is an instance of easyXML
        >>> # You can retreive XML from this instance of easyXML and print the content
        >>> # in a more human-readable way.
        >>> res.soup.findAll('id') # a Beautifulsoup instance is available
        >>> res.root # and the root using xml.etree.ElementTree

    There is a getitem so you can type::

        res['id']

    which is equivalent to::

        res.soup.findAll('id')

    There is also aliases findAll and prettify.

    """
    def __init__(self, data, fixing_unicode=False, encoding="utf-8"):
        """.. rubric:: Constructor

        :param data: an XML document format
        :param fixing_unicode: use only with HGNC service to fix issue with the
            XML returned by that particular service. No need to use otherwise.
            See :class:`~bioservices.hgnc.HGNC` documentation for details.
        :param encoding: default is utf-8 used. Used to fix the HGNC XML only.


        The data parameter must be a string containing the XML document. If you
        have an URL instead, use :class:`readXML`

        """
        if fixing_unicode:
            x = unicodefix.FixingUnicode(data, verbose=False, encoding=encoding)
            self.data = x.fixed_string.encode("utf-8")
        else:
            self.data = data[:]

        try:
            self.root = ET.fromstring(self.data)
        except:
            self.root = self.data[:]
        self._soup = None
        self.prettify = self.soup.prettify
        self.findAll = self.soup.findAll

    def getchildren(self):
        """returns all children of the root XML document

        This is just an alias to self.soup.getchildren()
        """
        return self.root.getchildren()

    def _get_soup(self):
        if self._soup == None:
            self._soup = bs4.BeautifulSoup(self.data)
        return self._soup
    soup = property(_get_soup, doc="Returns the beautiful soup instance")

    def __str__(self):
        txt = self.soup.prettify()
        return txt

    def __getitem__(self, i):
        return self.findAll(i)


class readXML(easyXML):
    """Read XML and converts to beautifulsoup data structure

    easyXML accepts as input a string. This class accepts a filename instead
    inherits from easyXML

    .. seealso:: :class:`easyXML`

    """
    def __init__(self, filename, fixing_unicode=False, encoding="utf-8"):
        url = urllib2.urlopen(filename, "r")
        self.data = url.read()
        super(readXML, self).__init__(self.data, fixing_unicode, encoding)






from bioservices.geneprof import GeneProf



def test_geneprof():
    g = GeneProf(verbose=False)
    #res = g.get_list_experiments(with_outputs=True)
    res = g.get_list_experiments(with_outputs=False)
    len(res)>0
    res = g.get_metadata_experiment("3")

    g.get_metadata_experiment(Id="385", with_workflow=True)
    g.get_metadata_dataset("11_12_122_1", with_ats=True)

def test_1():
    g = GeneProf(verbose=False)
    g.get_list_reference_datasets()

    g.get_list_experiment_samples("mouse")
    g.get_list_experiment_samples("human", format="txt")
    g.get_list_experiment_samples("human", format="rdata")

    g.search_genes("sox2")
    g.search_genes("sox2", taxons="9606")
    g.search_genes("sox2", taxons="9606, 10090")
    res = g.search_genes("brca2 AND cancer AND reference", taxons="mouse")
def test_2():
    g = GeneProf(verbose=False)
    g.search_experiments("sox2")
    g.search_experiments("citation:cancer")
    g.search_experiments("citation:'stem cell'")
    g.search_datasets('sox2')
    g.search_datasets('gene expression')
    g.search_datasets("datatype:GENOMIC_REGIONS AND sox2")
    g.search_samples("ChIP")
    g.search_samples("Gene:sox2")
    g.search_samples("human")

def test_3():
    g = GeneProf(verbose=False)
    g.get_gene_id("mouse", "C_ENSG", "ENSMUSG00000059552")
    g.get_gene_id("human", "C_RSEQ", "NM_005657")
    g.get_gene_id("human", "any", "NM_005657")
    g.get_external_gene_id("mouse","715", "C_ENSG")
    g.get_external_gene_id("mouse","2981", "C_RSEQ")
    g.get_external_gene_id("mouse","2981", "C_NAME")
    g.get_list_idtypes("mouse")
    g.get_list_idtypes("human")
    g.get_gene_expression("mouse", "715", with_sample_info=True)
    g.get_gene_expression("mouse", "715", type="RAW")
    g.get_gene_expression("mouse", "715", format="txt", with_sample_info=True)
    g.get_gene_expression("mouse", "715", format="rdata", with_sample_info=True)
   

def test_gettargets():

    g = GeneProf(verbose=False)
    g.get_targets_tf("mouse", "9885")
    g.get_targets_tf("human", "36958", include_unbound=True)
    g.get_targets_tf("mouse", "14899", ats="C_NAME,C_ENSG")
    g.get_targets_tf("human", "36958", format="rdata")
    g.get_targets_by_experiment_sample("mouse", "541")
    g.get_targets_by_experiment_sample("human", "784", include_unbound=True)
    g.get_targets_by_experiment_sample("mouse", "541", ats="C_NAME,C_ENSG")
    g.get_targets_by_experiment_sample("human", "784", format="rdata")
        
def test_get_tfas():
    g = GeneProf(verbose=False)
    g.get_tfas_by_gene("mouse", "9885")
    g.get_tfas_by_gene("human", "36958", format="xml",
                    include_unbound=True)
    g.get_tfas_by_gene("mouse", "14899", format="txt", 
                    ats="C_NAME,C_ENSG")
    g.get_tfas_by_gene("human", "36958", format="rdata")

    g.get_tfas_by_sample("mouse", 541)
    g.get_tfas_by_sample("human", "784", format="xml",
                    include_unbound=True)
    g.get_tfas_by_sample("mouse", "541", format="txt", 
                ats="C_NAME,C_ENSG")
    g.get_tfas_by_sample("human", "784")

def test_get_tf():
    g = GeneProf(verbose=False)
    g.get_tf_by_target_gene("mouse", "715", with_sample_info=True)
    g.get_tf_by_target_gene("mouse", "715", format="xml",
                    with_sample_info=True)
    g.get_tf_by_target_gene("mouse", "715", format="txt")
    g.get_tf_by_target_gene("mouse", "715", format="rdata", 
                    with_sample_info=True)

    
    g.get_tfas_scores_by_target("mouse", "715", with_sample_info=True)
    g.get_tfas_scores_by_target("mouse", 715, with_sample_info=True)
    g.get_tfas_scores_by_target("mouse", 715, format="txt")
    g.get_tfas_scores_by_target("mouse", "715", format="rdata",
                with_sample_info=True)

def test_data():
    g = GeneProf(verbose=False)
    g.get_data("11_119_18_1", format="txt", gz=True)
    g.get_data("11_119_18_1", format="txt", gz=True, 
         ats="C_ENSG,C_11_119_16_1_RPKM0,C_11_119_16_1_RPKM1,C_11_119_16_1_RPKM2,C_11_119_16_1_RPKM3")


    g.get_chromosome_names("pub_mm_ens58_ncbim37", format="txt")
    g.get_chromosome_names("pub_hs_ens59_grch37", format="json")
    g.get_chromosome_names("11_3_7_2", format="xml")

    g.get_bed_files("11_3_7_2")
    g.get_bed_files("11_3_7_2", chromosome="3-chr3")
    g.get_bed_files("zebrafish", with_track_description=False)
    g.get_bed_files("11_12_125_2", filter_column="C_11_12_125_2_14_TFBS")
    g.get_wig_files("11_58_16_2")
    g.get_wig_files("11_12_112_2", with_track_description=False,
                only_distinct=True, frag_length=200)
    
    g.get_fasta("11_385_6_1")
    g.get_fastq("11_385_6_1")




from bioservices import HGNC


class test_hgnc():

    def __init__(self):
        self.s = HGNC(verbose=False)

    def test_get_xml(self):
        xml = self.s.get_xml("ZAP70")
        xml = self.s.get_xml("ZAP70;INSR")
        assert len(xml.findAll("gene")) == 2
        self.s.get_xml("wrong")

    def test_aliases(self):
        assert self.s.get_aliases("ZAP70") == [u'ZAP-70', u'STD']
        self.s.get_name("ZAP70")
        self.s.get_chromosome("ZAP70")
        self.s.get_previous_symbols("ZAP70")
        self.s.get_withdrawn_symbols("ZAP70")
        self.s.get_previous_names("ZAP70")



    def test_xref(self):
        assert self.s.get_xrefs("ZAP70")['UniProt']['xkey'] == 'P43403'
        assert self.s.get_xrefs("ZAP70", "xml")['UniProt']['link'] == ['http://www.uniprot.org/uniprot/P43403.xml']

    def test_lookfor(self):    
        self.s.lookfor("ZAP70")

    def test_mapping(self):
        value = "UniProt:P43403"
        res = self.s.mapping(value)
        res[0]['xlink:title'] == "ZAP70"






from bioservices.xmltools import easyXML



xmldata = """
<xml>
  <parameter>
    <value>1</value>
  </parameter>
</xml>
"""


def test_easyXML():
    res = easyXML(xmldata)
    print res
    res['parameter']



from bioservices.apps.fasta import *
import tempfile


def test_fasta():
    f = FASTA()
    f.load_fasta("P43403")
    fasta = f.get_fasta("P43403")
    f.header
    f.gene_name
    f.sequence
    fh = tempfile.NamedTemporaryFile(delete=False)
    f.save_fasta(fasta, fh.name)
    f.read_fasta(fh.name)
    fh.delete = True
    fh.close()


def test_multi_fasta():
    f = MultiFASTA()
    f.load_fasta("P43403")
    f.load_fasta("P43408")
    f.sequences
    f.ids
    f.fasta

    fh = tempfile.NamedTemporaryFile(delete=False)
    f.save_fasta(fh.name)
    f.read_fasta(fh.name)

    f.fasta["P43403"]



# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: pdb.py 189 2013-04-16 16:58:11Z cokelaer $
"""Interface to the PDB web Service.

.. topic:: What is PDB ?

    :URL: http://www.rcsb.org/pdb/
    :REST: http://www.rcsb.org/pdb/software/rest.do

    .. highlights::

        An Information Portal to Biological Macromolecular Structures

        -- PDB home page, Feb 2013

    :Status: in progress not for production

"""
from __future__ import print_function

from bioservices.services import RESTService, BioServicesError

__all__ = ["PDB"]


class PDB(RESTService):
    """Interface to part of the `PDB <http://www.rcsb.org/pdb>`_ service

    :Status: in progress not for production. You can get all ID and retrieve
        uncompressed file in PDB/FASTA formats for now. New features will be
        added on request.

    .. doctest::

        >>> from bioservices import PDB
        >>> s = PDB()
        >>> res = s.getFile("1FBV", "pdb")

    """

    def __init__(self, verbose=False):
        """.. rubric:: Constructor

        :param bool verbose: prints informative messages (default is off)

        """
        super(PDB, self).__init__(name="PDB",
            url="http://www.rcsb.org/pdb/rest/", verbose=verbose)
        self.easyXMLConversion = True

    def getCurrentIDs(self):
        """Get a list of all current PDB IDs."""
        res = self.request("getCurrent")
        res = [x.attrib['structureId'] for x in res.getchildren()]
        return res

    def getFile(self, Id, fileFormat):
        """Download a file in a specified format

        :param int Id: a valid Identifier. See :meth:`getCurrentIDs`.
        :param str fileFormat: a valid format in "FASTA", "pdb", "cif", "xml"

        .. doctest::

            >>> from bioservices import PDB
            >>> s = PDB()
            >>> res = s.getFile("1FBV", "pdb")
            >>> import tempfile
            >>> fh = tempfile.NamedTemporaryFile()
            >>> fh.write(res)
            >>> # manipulate the PDB file with your favorite tool
            >>> # close the file ONLY when finished (this is temporary file)
            >>> # fh.close()


        """
        valid_formats = ["FASTA", "pdb", "cif", "xml"]
        self.checkParam(fileFormat, valid_formats)
        formatter = "download/downloadFile.do?fileFormat=%s&compression=NO&structureId=%s"
        if fileFormat == "xml":
            res = self.request(formatter % (fileFormat, Id))
        else:
            res = self.request(formatter % (fileFormat, Id), format="txt")
        return res




# -*- coding: utf-8 -*-
__revision__ = "$Id: setup.py 296 2013-12-10 21:34:29Z cokelaer $"
import sys
import os
from setuptools import setup, find_packages
import glob


_MAJOR               = 1
_MINOR               = 2
_MICRO               = 0
version              = '%d.%d.%d' % (_MAJOR, _MINOR, _MICRO)
release              = '%d.%d' % (_MAJOR, _MINOR)

metainfo = {
    'authors': {
        'Cokelaer':('Thomas Cokelaer','cokelaer@ebi.ac.uk'),
        },
    'version': version,
    'license' : 'GPL',
    'download_url' : ['http://pypi.python.org/pypi/bioservices'],
    'url' : ['http://pypi.python.org/pypi/bioservices'],
    'description':'Access to Biological Web Services from Python' ,
    'platforms' : ['Linux', 'Unix', 'MacOsX', 'Windows'],
    'keywords' : [
        "BioServices", "Web Services", "Biology", "ChEBI", "UniChem", "Kegg", "KEGG", "BioModels",
        "EUtils", "UniProt", "PICR", "ArrayExpress", "MUSCLE", "QuickGO", "PDB", "PSICQUIC", "Blast",
        "BioMART", "BioGRID", "MIRIAM", "BioMart", "GeneProf", "EUtils", "ChEMBL", "ChemSpider", 
        "HGNC", "PathwayCommons"],
    'classifiers' : [
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.7',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Topic :: Scientific/Engineering :: Bio-Informatics',
          'Topic :: Scientific/Engineering :: Information Analysis',
          'Topic :: Scientific/Engineering :: Mathematics',
          'Topic :: Scientific/Engineering :: Physics']
    }



setup(
    name             = 'bioservices',
    version          = version,
    maintainer       = metainfo['authors']['Cokelaer'][0],
    maintainer_email = metainfo['authors']['Cokelaer'][1],
    author           = metainfo['authors']['Cokelaer'][0],
    author_email     = metainfo['authors']['Cokelaer'][1],
    long_description = open("README.txt").read(),
    keywords         = metainfo['keywords'],
    description = metainfo['description'],
    license          = metainfo['license'],
    platforms        = metainfo['platforms'],
    url              = metainfo['url'],      
    download_url     = metainfo['download_url'],
    classifiers      = metainfo['classifiers'],

    # package installation
    package_dir = {'':'src'},
    packages = ['bioservices'],
    #package_dir  = package_dir,
    install_requires = ["easydev>=0.5.6", "beautifulsoup4", "SOAPpy", "suds"],
    )





#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EMBL-EBI
#
#  File author(s):
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: xmltools.py 230 2013-07-09 17:12:59Z cokelaer $
"""This module includes common tools to manipulate HTML files"""
from __future__ import print_function
from HTMLParser import HTMLParser

__all__ = ["easyHTML"]




class easyHTML(HTMLParser):
    """class to ease the introspection of XML documents.


    """
    def __init__(self, url=None, data=None, filename=None):
        """.. rubric:: Constructor

        :param data: an XML document format
        :param fixing_unicode: use only with HGNC service to fix issue with the
            XML returned by that particular service. No need to use otherwise.
            See :class:`~bioservices.hgnc.HGNC` documentation for details.
        :param encoding: default is utf-8 used. Used to fix the HGNC XML only.

        """
        # cannot use super with HTMLParser
        #super(easyHTML, self).__init__()
        HTMLParser.__init__(self)
        if url and data or url and filename or filename and data:
            raise ValueError("only 1 argument must be provided")
        if url:
            import urllib2
            html = urllib2.urlopen(url, "r")
            self.data = html.read()
        elif data:
            self.data = data[:]
        elif filename:
            self.data = open(filename, "r").read()
        else:
            raise ValueError("at least 1 argument must be provided")

        self.feed(self.data)






from bioservices import BioModels


modelId = 'BIOMD0000000256'
uniprotId = 'P10113'
pubId = '18308339'
GOId = 'GO:0001756'
reacID = "REACT_1590" 
personName = "LeNovere"

class test_biomodels(BioModels):

    def __init__(self):
        super(test_biomodels, self).__init__(verbose=False)

    def test_size(self):

        L1 = len(self.getAllCuratedModelsId())
        L2 = len(self.getAllNonCuratedModelsId())
        L = len(self)
        assert L == L1+L2


    def test_getAllModelsId(self):
        assert len(self.getAllModelsId())>800

    def test_getAllCuratedModelsId(self):
        assert len(self.getAllCuratedModelsId())>100

    def test_getAllNonCuratedModelsId(self):
        assert len(self.getAllNonCuratedModelsId())>100

    def test_getModelById(self):
        self.getModelById('MODEL1006230101')

    def test_getModelSBMLById(self):
        self.getModelSBMLById(modelId)
        self.getModelSBMLById('MODEL1006230101')

    def test_getAuthorsByModelId(self):
        res = self.getAuthorsByModelId(modelId)
        assert res == ['Rehm M', 'Huber HJ', 'Dussmann H', 'Prehn JH']

    def test_getDateLastModifByModelId(self):
        res = self.getDateLastModifByModelId(modelId)
        assert res == '2012-05-16T14:44:17+00:00'

    def test_getEncodersByModelId(self):
        res = self.getEncodersByModelId("BIOMD0000000256")
        assert res == ['Lukas Endler']

    def test_getLastModifiedDateByModelId(self):
        res = self.getLastModifiedDateByModelId("BIOMD0000000256")
        assert res == '2012-05-16T14:44:17+00:00'

    def test_getModelNameById(self):
        res = self.getModelNameById("BIOMD0000000256")
        assert res == 'Rehm2006_Caspase'

        try:
            self.getModelNameById("dummy")
            assert False
        except:assert True

    def test_getModelsIdByChEBI(self):
        res =  self.getModelsIdByChEBI('CHEBI:4978')
        res == ['BIOMD0000000217', 'BIOMD0000000404']

    def test_getModelsIdByChEBIId(self):
        res = self.getModelsIdByChEBIId('CHEBI:4978')
        assert res == ['BIOMD0000000404']

    def test_getSimpleModelsByChEBIIds(self):
        self.getSimpleModelsByChEBIIds('CHEBI:4978')

    def test_getSimpleModelsRelatedWithChEBI(self):
        res = self.getSimpleModelsRelatedWithChEBI()
        from bioservices import xmltools
        res = xmltools.easyXML(res.encode('utf-8'))    
        modelIDs = set([x.findall('modelId')[0].text for x in res.getchildren()])
        assert len(modelIDs) > 1


    
    def test_getPublicationByModelId(self):
        res = self.getPublicationByModelId("BIOMD0000000256")
        assert res == '16932741'


    def test_getSimpleModelByIds(self):
        self.getSimpleModelsByIds(modelId)

    def test_getModelsIdByPerson(self):
        self.getModelsIdByPerson(personName)

    def test_getSimpleModelsByReactomeIds(self):
        return self.getSimpleModelsByReactomeIds(reacID)

    def test_getModelsIdByUniprotId(self):
        return self.getModelsIdByUniprotId(uniprotId)

    def test_getModelsIdByUniprotIds(self):
        self.getModelsIdByUniprotIds(["P10113", "P10415"])

    def test_getModelsIdByName(self):
        return self.getModelsIdByName('2009')

    def test_getModelsIdByPublication(self):
        res = self.getModelsIdByPublication(pubId)
        assert res == ['BIOMD0000000201']

    def test_getModelsIdByGO(self):
        return self.getModelsIdByGO(GOId)

    def test_getModelsIdByTaxonomy(self):
        return self.getModelsIdByTaxonomy("EGF")

    def test_getModelsIdByTaxonomyId(self, taxonomyId='9606'):
        return self.getModelsIdByTaxonomyId(taxonomyId)

    def test_getSubModelSBML(self):
        self.getSubModelSBML("BIOMD0000000242", "cyclinEdegradation_1")
        #self.getSubModelSBML(modelId, elementsIDs)
        pass

    def test_getModelsIdByGOId(self):
        self.getModelsIdByGOId(GOId)

    def test_extra_getChEBIIds(self):
        self.extra_getChEBIIds(99, 101)
        try:
            self.extra_getChEBIIds(1000,101)
            assert False
        except:
            assert True

    def test_extra_getReactomeIds(self):
        self.extra_getReactomeIds(99,101)  # just to cross the 100 Ids
        self.extra_getReactomeIds(89,90)  # just to get one output REACT_89
        try:
            self.extra_getReactomeIds(1000,101)
            assert False
        except:
            assert True

    def test_extra_getUniprotIds(self):
        self.extra_getUniprotIds(11099,11101)
        self.extra_getUniprotIds(10113,10114)

        try:
            self.extra_getUniprotIds(1000,101)
            assert False
        except ValueError:
            assert True



    def test_getModelsIdByUniprot(self):
        self.getModelsIdByUniprot("P10113")
